#!/usr/bin/env python
import math
import re
import shlex
import subprocess
import time


class Pos:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __str__(self):
        return "%s %s" % (str(self.x), str(self.y))

    def distance(self, other):
        dx = self.x - other.x
        dy = self.y - other.y
        return math.sqrt(dx * dx + dy * dy)


def run_command(command):
    state = {
        "pressed": False,
        "pressPos": None,
        "pressTime": None,
        "lastEventTime": None,
        "lastPos": None,
        "danglingUp": False,
    }
    pattern = re.compile("\\[\\s+(\\d+\\.\\d+)\\]\\s*[\\w/]+:\\s*(\\w+)\\s+(\\w+)\\s+(\\w+)")

    process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE)
    while True:
        line = process.stdout.readline()
        if line == '' and process.poll() is not None:
            break
        line = line.strip()
        match = pattern.match(line)
        if not match:
            continue

        time = float(match.group(1))
        evType = match.group(2)
        evAction = match.group(3)
        evParam = match.group(4)

        def finishTouchEvent():
            moveDeltaPx = state["lastPos"].distance(state["pressPos"])
            timeDeltaMs = int(1000 * (time - state["pressTime"]))

            # print "distance: %s, time: %f" % (moveDeltaPx, timeDeltaMs)

            if moveDeltaPx < 50:
                if timeDeltaMs < 300:
                    print "tap %s" % state["pressPos"]
                else:
                    print "longPress %s %s" % (state["pressPos"], timeDeltaMs)
            else:
                print "swipe %s %s %d" % (state["pressPos"], state["lastPos"], timeDeltaMs)

            state["lastEventTime"] = time

        if evAction == "BTN_TOUCH":
            if evParam == "DOWN":
                # print "press " + str(time)
                state["pressed"] = True
                state["pressPos"] = None
                state["pressTime"] = time
                if state["lastEventTime"] is not None:
                    print "sleep %d" % int(1000 * (time - state["lastEventTime"]))
            elif evParam == "UP":
                # print "release " + str(time)
                state["pressed"] = False
                if state["lastPos"].y is None:
                    state["danglingUp"] = True
                else:
                    # workaround for changed output orders
                    if state["pressPos"] == None:
                        state["pressPos"] = state["lastPos"]
                    finishTouchEvent()

                # print "lastPos: %s, pressPos: %s" % (state["lastPos"], state["pressPos"])

            else:
                raise Exception("unknown BTNTOUCH " + evParam)

        elif evAction == "ABS_MT_POSITION_X":
            state["lastPos"] = Pos(int(evParam, 16), None)
            if state["pressPos"] == None:
                state["pressPos"] = state["lastPos"]

        elif evAction == "ABS_MT_POSITION_Y":
            state["lastPos"].y = int(evParam, 16)

            if state["danglingUp"]:
                if state["pressPos"] is None or state["pressPos"].y is None:
                    state["pressPos"] = state["lastPos"]

                state["danglingUp"] = False
                finishTouchEvent()

        elif evType == "EV_KEY":
            if evParam == "UP":
                code = "KEYCODE_" + evAction[4:]
                print "keyEvent " + code

    rc = process.poll()
    return rc


def main():
    while True:
        # Command somehow sometimes finishes ... USB-Connection?
        run_command("adb shell getevent -lt")
        time.sleep(0.5)


if __name__ == "__main__":
    main()
