#!/usr/bin/env python

import argparse
import json
import urllib2
from bs4 import BeautifulSoup
from os import path

APK_PURE_BASE_URL = "https://apkpure.com"
APK_PURE_APP_URL = APK_PURE_BASE_URL + "/store/apps/details?id="


def readFile(path):
    with open(path, "r") as f:
        return f.read()


def writeFile(path, content):
    with open(path, "w") as f:
        return f.write(content)


def download(url):
    opener = urllib2.build_opener()
    # set user-agent, otherwise apkpure will reject query with 403 forbidden
    opener.addheaders = [('User-Agent', 'Mozilla/5.0')]
    response = opener.open(url)
    return response.read()


def downloadApk(package, filePath):
    if path.exists(filePath):
        print "file " + filePath + " already exists"
        return

    page = download(APK_PURE_APP_URL + package)
    soup = BeautifulSoup(page, 'html.parser')
    downloadLink = APK_PURE_BASE_URL + soup.findAll("a", {"class": "da"})[0].attrs["href"]
    page = download(downloadLink)
    soup = BeautifulSoup(page, 'html.parser')
    realDownloadLink = soup.find(id="download_link").attrs["href"]
    apkContent = download(realDownloadLink)
    writeFile(filePath, apkContent)

def main(appIndex, apkDir):
    top100 = json.loads(readFile(appIndex))
    result = []
    num = 1
    for idx, app in enumerate(top100):
        if num > 100: break
        assert idx+1 == int(app["number"])

        try:
            fileName = "%03d_%s.apk" % (num, app["package"])
            filePath = path.join(apkDir, fileName)
            print "download apk " + fileName
            downloadApk(app["package"], filePath)
            result.append({"number": num, "package": app["package"]})
            num += 1
        except:
            print "failed to download " + app["package"] + " skip"

    writeFile(appIndex, json.dumps(result, indent=2))


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Download first 100 apk files available on apkpure")
    parser.add_argument("appIndex", help="Json containing list of top 100 apks")
    parser.add_argument("apkDir", help="Folder where to store apks")
    args = parser.parse_args()
    main(args.appIndex, args.apkDir)
