package de.marstra.dvo.evaluation;

import joptsimple.OptionParser;
import joptsimple.OptionSet;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;


@Slf4j
public class Main {
    private static OptionSet parseArgs(String args[]) {
        // see usage of options parser: https://pholser.github.io/jopt-simple/examples.html
        OptionParser parser = new OptionParser();
        parser.accepts("top100", "analyze top 100 play apks");
        parser.accepts("benchmarkApps", "analyze benchmark apps");
        parser.accepts("algos", "analyze benchmark algos");

        parser.acceptsAll(Arrays.asList("i", "index"), "index json")
                .requiredIf("top100", "benchmarkApps", "algos")
                .withRequiredArg().ofType(File.class);

        parser.acceptsAll(Arrays.asList("af", "apkFolder"), "folder where apks are stored")
                .requiredIf("top100", "benchmarkApps")
                .withRequiredArg().ofType(File.class);

        parser.acceptsAll(Arrays.asList("r", "record"), "record input sequences")
                .availableIf("benchmarkApps");

        parser.acceptsAll(Arrays.asList("o", "only"), "evaluate only provided app")
                .availableIf("top100", "benchmarkApps", "algos")
                .withRequiredArg()
                .describedAs("app1,app2,...")
                .withValuesSeparatedBy(",")
                .ofType(String.class);

        parser.acceptsAll(Arrays.asList("h", "?", "help"), "show help").forHelp();
        return parser.parse(args);
    }

    public static void main(String args[]) throws IOException {
        OptionSet options = parseArgs(args);

        if (options.has("top100")) {
            PlayTop100.evaluateApks(
                    (File) options.valueOf("index"),
                    (File) options.valueOf("apkFolder"),
                    (List<String>) options.valuesOf("only")
            );
        }
        else if (options.has("benchmarkApps")) {
            AppBenchmark.benchmarkApps(
                    (File) options.valueOf("index"),
                    (File) options.valueOf("apkFolder"),
                    (List<String>) options.valuesOf("only"),
                    options.has("record")
            );
        }
        else if (options.has("algos")) {
            AlgoBenchmark.benchmarkAlgos(
                    (File) options.valueOf("index"),
                    (List<String>) options.valuesOf("only")
            );
        }
        else {
            System.err.println("usage: <" + Main.class.getName() + "> [top100|benchmarkApps|algos] <args>");
            System.exit(1);
        }
    }
}
