package de.marstra.dvo.evaluation;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonStreamContext;
import com.fasterxml.jackson.core.util.DefaultPrettyPrinter;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class MyJsonPrettyPrinter extends DefaultPrettyPrinter {
    public MyJsonPrettyPrinter() {
        this(new MyArrayIndenter());
    }

    public MyJsonPrettyPrinter(DefaultPrettyPrinter.Indenter arrayIndenter) {
        this._arrayIndenter = arrayIndenter;
        this._objectFieldValueSeparatorWithSpaces = this._separators.getObjectFieldValueSeparator() + " ";
    }

    public static class MyArrayIndenter extends DefaultPrettyPrinter.FixedSpaceIndenter {
        private final Set<String> inline;

        public MyArrayIndenter(List<String> inline) {
            this.inline = new HashSet<>(inline);
        }

        public MyArrayIndenter(String... inline) {
            this(Arrays.asList(inline));
        }

        @Override
        public void writeIndentation(JsonGenerator g, int level) throws IOException {
            JsonStreamContext ctx = g.getOutputContext();
            if (ctx != null) {
                JsonStreamContext parent = ctx.getParent();
                if (parent != null && this.inline.contains(parent.getCurrentName())) {
                    g.writeRaw(' ');
                    return;
                }
            }

            StringBuilder spaces = new StringBuilder();
            for (int i = 0; i < level; i++) {
                spaces.append("  ");
            }
            g.writeRaw("\n" + spaces);
        }

        @Override
        public boolean isInline() {
            return false;
        }
    }
}
