package de.marstra.dvo.evaluation;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.marstra.dvo.obfuscator.Config;
import de.marstra.dvo.obfuscator.InterpreterGenerator;
import de.marstra.dvo.obfuscator.analysis.AnalysisVisitorAdapter;
import de.marstra.dvo.obfuscator.analysis.DexMethod;
import de.marstra.dvo.obfuscator.analysis.MethodAnalysis;
import de.marstra.dvo.obfuscator.analysis.ParsedDex;
import de.marstra.dvo.obfuscator.util.GenericMethodReference;
import org.apache.commons.io.IOUtils;

import java.io.*;
import java.net.URL;
import java.nio.file.Path;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.BiFunction;

public class Utils {
    public static void download(String url, Path file) throws IOException {
        InputStream in = new URL(url).openStream();
        OutputStream out = new FileOutputStream(file.toFile());
        IOUtils.copy(in, out);
    }

    public static Config generateConfig(ParsedDex parsedDex, BiFunction<DexMethod, ParsedDex, Boolean> isExcluded) {
        return generateConfig(parsedDex, isExcluded, new HashSet<>(), true, false, false, false);
    }

    public static Config generateConfig(ParsedDex parsedDex, BiFunction<DexMethod, ParsedDex, Boolean> isExcluded,
                                        Set<InterpreterGenerator.VerbosityLevel> interpreterVerbosityLevels, boolean signDebug,
                                        boolean protect, boolean debugProtection, boolean renameInterpreter) {
        final BiFunction<DexMethod, ParsedDex, Boolean> filter = isExcluded != null ? isExcluded :
                ((dm, pd) -> !MethodAnalysis.analyseMethod(dm.getMethod(), pd).isSupported());
        Config config = new Config(new HashSet<>(), interpreterVerbosityLevels, signDebug, protect, debugProtection, renameInterpreter);
        parsedDex.accept(new AnalysisVisitorAdapter<Void, Void>() {
            @Override
            public Void visit(DexMethod dexMethod, Void p) {
                if (!filter.apply(dexMethod, parsedDex)) {
                    config.getMethodsToObfuscate().add(new GenericMethodReference(dexMethod.getMethod()));
                }
                return null;
            }
        });
        return config;
    }

    public static void sleep(int time) {
        try {
            Thread.sleep(time);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public static void serializeJson(ObjectMapper mapper, File target, Object object) {
        serializeJson(mapper, target, object, Collections.emptyList());
    }

    public static void serializeJson(ObjectMapper mapper, File target, Object object, List<String> inline) {
        try {
            mapper.writer(new MyJsonPrettyPrinter(new MyJsonPrettyPrinter.MyArrayIndenter(inline))).writeValue(target, object);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static Integer toInt(String s) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            return null;
        }
    }
}
