package de.marstra.dvo.obfuscator;

import de.marstra.dvo.obfuscator.insn.InstructionType;

import java.util.*;
import java.util.function.Function;
import java.util.stream.IntStream;

public class ByteCodeMapping {
    private final Map<InstructionType, Byte> instructionTypetoOp;
    private final Map<Byte, InstructionType> opToInstructionType;

    private ByteCodeMapping(Map<InstructionType, Byte> instructionTypetoOp, Map<Byte, InstructionType> opToInstructionType) {
        this.instructionTypetoOp = instructionTypetoOp;
        this.opToInstructionType = opToInstructionType;
    }

    public byte toVmOpcode(InstructionType instructionType) {
        return instructionTypetoOp.get(instructionType);
    }

    public InstructionType toInstructionType(byte op) {
        return opToInstructionType.get(op);
    }

    private static ByteCodeMapping getMapping(Function<Integer, Integer> selector) {
        Map<InstructionType, Byte> instructionTypetoOp = new HashMap<>();
        Map<Byte, InstructionType> opToInstructionType = new HashMap<>();
        List<Byte> remaining = new LinkedList<>();
        IntStream.rangeClosed(0, 255).forEach(i -> remaining.add((byte) i));

        for (InstructionType instructionType : InstructionType.values()) {
            byte opcode = remaining.remove((int) selector.apply(remaining.size()));
            instructionTypetoOp.put(instructionType, opcode);
            opToInstructionType.put(opcode, instructionType);
        }
        return new ByteCodeMapping(instructionTypetoOp, opToInstructionType);
    }

    public static ByteCodeMapping getRandomMapping() {
        return getMapping((size) -> new Random().nextInt(size));
    }

    public static ByteCodeMapping getDefaultMapping() {
        return getMapping((size) -> 0);
    }
}
