package de.marstra.dvo.obfuscator;

import de.marstra.dvo.obfuscator.analysis.ApkFile;
import joptsimple.OptionParser;
import joptsimple.OptionSet;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Arrays;

public class Main {
    private static void printHelp(OptionParser parser) {
        try {
            parser.printHelpOn(System.out);
        } catch (IOException e1) {
            throw new RuntimeException(e1);
        }
    }

    private static OptionSet parseArgs(String args[]) {
        // see usage of options parser: https://pholser.github.io/jopt-simple/examples.html
        OptionParser parser = new OptionParser();
        parser.acceptsAll(Arrays.asList("in", "input"), "input apk/dex").withRequiredArg().required().ofType(File.class);
        parser.acceptsAll(Arrays.asList("out", "output"), "output apk/dex").withRequiredArg().required().ofType(File.class);
        parser.acceptsAll(Arrays.asList("c", "config"), "config file").withRequiredArg().required().ofType(File.class);
        parser.acceptsAll(Arrays.asList("h", "?", "help"), "show help").forHelp();

        try {
            OptionSet options = parser.parse(args);

            if (options.has("help")) {
                printHelp(parser);
                System.exit(0);
            }

            return options;
        } catch (Exception e) {
            printHelp(parser);
            System.exit(1);
            throw new IllegalStateException("Program should exit");
        }
    }

    public static void main(String args[]) throws IOException {
        OptionSet options = parseArgs(args);
        File inFile = (File) options.valueOf("input");
        File outFile = (File) options.valueOf("output");
        File configFile = (File) options.valueOf("config");

        Config config = Config.fromFile(configFile);
        ApkFile inApk = new ApkFile(inFile);
        byte[] result = ObfuscatorService.obfuscate(inApk, config);
        Files.write(outFile.toPath(), result);
    }
}
