package de.marstra.dvo.obfuscator;

import de.marstra.dvo.obfuscator.analysis.ApkFile;
import de.marstra.dvo.obfuscator.analysis.ParsedDex;
import de.marstra.dvo.obfuscator.util.GenericMethodReference;
import interpreter.Utils;
import org.jf.dexlib2.iface.DexFile;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static de.marstra.dvo.obfuscator.InterpreterGenerator.VMODEX_FILE;

public class ObfuscatorService {
    /**
     * Patch provided apk file to replace all methods specified in config with calls to interpreter
     *
     * @param apk    original apk to patch
     * @param config configuration which methods should be patched
     * @return patched apk file as byte array
     */
    public static byte[] obfuscate(ApkFile apk, Config config) {
        ParsedDex parsedDex = apk.getParsedDex();
        Set<GenericMethodReference> toObfuscate = new HashSet<>(config.getMethodsToObfuscate());
        List<String> checksumLocations = null;

        if (config.isProtect()) {
            // TODO singing key as parameter?
            if (!config.isSignDebug())
                throw new IllegalStateException("protection measures only supported when app is signed");
            // TODO num checksum files as parameter
            checksumLocations = ProtectionInjector.getChecksumLocations(apk, 5);
            List<GenericMethodReference> protMethods = ProtectionInjector.addProtectionMeasures(parsedDex, apk.getEntryPoints(), toObfuscate, checksumLocations, config.isDebugProtection());
            toObfuscate.addAll(protMethods);
        }
        byte[] encryptionKey = Utils.getRandomBytes(16);

        ByteCodeMapping mapping = ByteCodeMapping.getRandomMapping();
        VMODexCompiler.CompilationResult compilationResult = VMODexCompiler.compile(parsedDex, toObfuscate, mapping);
        DexFile interpreter = InterpreterGenerator.generateInterpreterDex(mapping, compilationResult.getUsedInstructionTypes(), config.getInterpreterVerbosityLevels(), encryptionKey, false);
        StubInjector.injectInterpreter(parsedDex, compilationResult.getConvertedMethodReferences(), interpreter, config.isRenameInterpreter());

        // replace orginal classes.dex and insert vmodex file
        apk.setDexFiles(parsedDex.toDexFiles());
        apk.setEntry(VMODEX_FILE, compilationResult.getVmoDex().serialize(encryptionKey), true);

        byte[] result = config.isSignDebug() ? apk.sign() : apk.serialize();

        if (config.isProtect()) {
            return ProtectionInjector.addChecksumFiles(result, checksumLocations);
        }
        return result;
    }
}
