package de.marstra.dvo.obfuscator.analysis;

import de.marstra.dvo.obfuscator.util.GenericMethodReference;
import org.jf.dexlib2.DexFileFactory;
import org.jf.dexlib2.iface.ClassDef;
import org.jf.dexlib2.iface.DexFile;
import org.jf.dexlib2.iface.Method;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

public class CallerSensitiveAnalysis {
    private static Set<GenericMethodReference> libCallerSensitiveMethods;

    public static Set<GenericMethodReference> getLibCallerSensitiveMethods() {
        if (libCallerSensitiveMethods == null) {
            libCallerSensitiveMethods = new HashSet<>();
            InputStream in = CallerSensitiveAnalysis.class.getResourceAsStream("/CallerSensitiveMethodsRT.txt");
            Scanner scanner = new Scanner(in);
            while (scanner.hasNextLine()) {
                String line = scanner.nextLine();
                if (line.trim().isEmpty()) continue;
                GenericMethodReference gmr = new GenericMethodReference(line.trim().replaceAll("\n", ""));
                libCallerSensitiveMethods.add(gmr);
            }
        }
        return libCallerSensitiveMethods;
    }

    public static Set<GenericMethodReference> getCallerSensitiveMethods(List<DexFile> dexFiles) {
        Set<GenericMethodReference> callerSensitive = new HashSet<>();
        for (DexFile dexFile : dexFiles) {
            for (ClassDef cd : dexFile.getClasses()) {
                for (Method m : cd.getMethods()) {
                    if (m.getAnnotations().stream().anyMatch(a -> a.getType().equals("Lsun/reflect/CallerSensitive;")))
                        callerSensitive.add(new GenericMethodReference(m));
                }
            }
        }
        return callerSensitive;
    }

    public static void main(String[] args) throws IOException {
        if (args.length < 1) {
            System.err.println("usage: <" + CallerSensitiveAnalysis.class.getSimpleName() + "> <dexfiles>");
            System.exit(1);
        }
        List<String> files = Arrays.asList(args);
        List<DexFile> dexFiles = files.stream().map(f -> {
            try {
                return DexFileFactory.loadDexFile(f, null);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }).collect(Collectors.toList());
        Set<GenericMethodReference> callerSensitives = getCallerSensitiveMethods(dexFiles);
        callerSensitives.forEach(System.out::println);
    }
}
