package de.marstra.dvo.obfuscator.analysis;

import com.google.common.collect.Sets;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import de.marstra.dvo.obfuscator.util.GenericMethodReference;
import org.jf.dexlib2.Opcode;
import org.jf.dexlib2.iface.instruction.Instruction;
import org.jf.dexlib2.iface.instruction.ReferenceInstruction;
import org.jf.dexlib2.iface.reference.MethodReference;

import java.util.Set;

import static org.jf.dexlib2.Opcode.*;

public class InstructionAnalysis {
    private static final Set INVOKE_OPS = Sets.newHashSet(INVOKE_VIRTUAL,
            INVOKE_SUPER,
            INVOKE_DIRECT,
            INVOKE_STATIC,
            INVOKE_INTERFACE,
            INVOKE_VIRTUAL_RANGE,
//            INVOKE_SUPER_RANGE,
            INVOKE_DIRECT_RANGE,
//            INVOKE_STATIC_RANGE,
            INVOKE_INTERFACE_RANGE,
// Odex invokes currently not supported anyway
//            INVOKE_DIRECT_EMPTY,
//            INVOKE_OBJECT_INIT_RANGE,
//            INVOKE_VIRTUAL_QUICK,
//            INVOKE_VIRTUAL_QUICK_RANGE,
//            INVOKE_SUPER_QUICK,
//            INVOKE_SUPER_QUICK_RANGE,
            INVOKE_POLYMORPHIC,
            INVOKE_POLYMORPHIC_RANGE,
            INVOKE_CUSTOM,
            INVOKE_CUSTOM_RANGE
    );

    private static boolean isInvokeInstruction(Instruction ins) {
        return INVOKE_OPS.contains(ins.getOpcode());
    }

    private static GenericMethodReference getInvocationTarget(Instruction ins) {
        if (!isInvokeInstruction(ins))
            throw new IllegalArgumentException("Given instruction: '" + ins.getOpcode() + "' is no invoke-instruction.");
        return new GenericMethodReference((MethodReference) ((ReferenceInstruction) ins).getReference());
    }

    public static String whyIsInstructionUnSupported(Instruction ins, ParsedDex parsedDex) {
        Opcode op = ins.getOpcode();
        if (op == MONITOR_ENTER || op == MONITOR_EXIT)
            return MethodAnalysis.MONITOR_INSTRUCTION;

        if (op == INVOKE_SUPER || op == INVOKE_SUPER_RANGE)
            return MethodAnalysis.INVOKE_SUPER;

        if (!VMODexCompiler.isInstructionSupported(ins))
//            return MethodAnalysis.UNSUPPORTED_INS + ": " + op;
            return MethodAnalysis.UNSUPPORTED_INS;

        if (isInvokeInstruction(ins)) {
            GenericMethodReference invocationTarget = getInvocationTarget(ins);
            if (parsedDex.isCallerSensitive(invocationTarget))
//                return MethodAnalysis.CALLER_SENSITIVE + ": " + ins.getOpcode() + "(" + invocationTarget + ")";
                return MethodAnalysis.CALLER_SENSITIVE;
        }
        return null;
    }
}
