package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.lang.reflect.Array;

import static de.marstra.dvo.obfuscator.insn.InstructionType.AGET;
import static de.marstra.dvo.obfuscator.insn.InstructionType.APUT;

public class ArrayLoadStore extends Instruction {
    private final int value;
    private final int array;
    private final int index;

    public ArrayLoadStore(InstructionType instructionType, int value, int array, int index) {
        super(instructionType, AGET, APUT);
        this.value = value;
        this.array = array;
        this.index = index;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return new byte[]{(byte) array, (byte) value, (byte) index};
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        boolean set = (boolean) type.getGenerateArgs()[0];
        builder.addStatement("Object array = regs[$L]", readUnsignedByte())
                .addStatement("int regNum = $L", readUnsignedByte())
                .addStatement("int index = ((InterpreterNumber) regs[$L]).intValue()", readUnsignedByte())
                // actual type is unimportant in non primitve case, its just necessary to know its an object type
                // so vmTypeToExternalType works properly and converts InterpreterNumber(0) to null
                .addStatement("Class componentType = array.getClass().getComponentType()")
                .addStatement("String typeName = componentType.isPrimitive() ? Utils.primitiveNameToType(componentType.getName()) : \"L\"");
        if (set) {
            builder.addStatement("Object nativeVal = Utils.vmTypeToExternalType(typeName, regs[regNum])")
                    .addStatement("$T.set(array, index, nativeVal)", Array.class);
        } else {
            builder.addStatement("Object nativeVal = $T.get(array, index)", Array.class)
                    .addStatement("regs[regNum] = Utils.externalTypeToVmType(typeName, nativeVal)");
        }
        builder.addStatement("break");
    }
}
