package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class BinOp extends Instruction {
    private final int target;
    private final int op1;
    private final int op2;

    public BinOp(InstructionType instructionType, int target, int op1, int op2) {
        super(instructionType, ADD_INT, SUB_INT, MUL_INT, DIV_INT, REM_INT, AND_INT, OR_INT, XOR_INT, SHL_INT, SHR_INT, USHR_INT, ADD_LONG, SUB_LONG, MUL_LONG, DIV_LONG, REM_LONG, AND_LONG, OR_LONG, XOR_LONG, SHL_LONG, SHR_LONG, USHR_LONG, ADD_FLOAT, SUB_FLOAT, MUL_FLOAT, DIV_FLOAT, REM_FLOAT, ADD_DOUBLE, SUB_DOUBLE, MUL_DOUBLE, DIV_DOUBLE, REM_DOUBLE, CMPL_FLOAT, CMPG_FLOAT, CMPL_DOUBLE, CMPG_DOUBLE, CMP_LONG);
        this.target = target;
        this.op1 = op1;
        this.op2 = op2;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(3)
                .put((byte) target)
                .put((byte) op1)
                .put((byte) op2)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        String conversion = type.getGenerateArgs()[0] + "Value()";
        String operation = (String) type.getGenerateArgs()[1];
        String val = "((InterpreterNumber) regs[$L])." + conversion;
        if (operation.contains("cmp")) {
            String opType = (String) type.getGenerateArgs()[0];
            opType = Character.toString(opType.charAt(0)).toUpperCase() + opType.substring(1);

            builder.addStatement("int dst = $L", readUnsignedByte())
                    .addStatement(opType + " v1 = " + val, readUnsignedByte())
                    .addStatement(opType + " v2 = " + val, readUnsignedByte());

            if (!operation.equals("cmp")) {
                builder.beginControlFlow("if (v1.isNaN() || v2.isNaN())")
                        .addStatement("regs[dst] = " + (operation.endsWith("g") ? "InterpreterNumber.ONE" : "InterpreterNumber.MINUS_ONE"))
                        .addStatement("break")
                        .endControlFlow();
            }
            builder.beginControlFlow("if (v1.equals(v2))")
                    .addStatement("regs[dst] = InterpreterNumber.ZERO")
                    .nextControlFlow("else")
                    .addStatement("regs[dst] = v1 > v2 ? InterpreterNumber.ONE : InterpreterNumber.MINUS_ONE")
                    .endControlFlow();
        } else {
            builder.addStatement("regs[$L] = new InterpreterNumber(" + val + " " + operation + " " + val + ")",
                    readUnsignedByte(), readUnsignedByte(), readUnsignedByte());
        }
        builder.addStatement("break");
    }
}

