package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class BinOpLit extends Instruction {
    private final int target;
    private final int source;
    private final int lit;

    public BinOpLit(InstructionType instructionType, int target, int source, int lit) {
        super(instructionType, ADD_INT_LIT, RSUB_INT_LIT, MUL_INT_LIT, DIV_INT_LIT, REM_INT_LIT, AND_INT_LIT, OR_INT_LIT, XOR_INT_LIT, SHL_INT_LIT, SHR_INT_LIT, USHR_INT_LIT);
        this.target = target;
        this.source = source;
        this.lit = lit;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(4).put((byte) target)
                .put((byte) source)
                .putShort((short) lit)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        String operation = (String) type.getGenerateArgs()[0];
        boolean reverse = false;
        if (type.getGenerateArgs().length > 1)
            reverse = (boolean) type.getGenerateArgs()[1];

        builder.addStatement("int target = $L", readUnsignedByte())
                .addStatement("int source = ((InterpreterNumber) regs[$L]).intValue()", readUnsignedByte())
                .addStatement("short lit = $L", readShort());

        final String code = reverse ?
                "regs[target] = new InterpreterNumber(lit " + operation + " source)" :
                "regs[target] = new InterpreterNumber(source " + operation + " lit)";

        builder.addStatement(code)
                .addStatement("break");
    }
}

