package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class Branch extends OffsetInstruction {
    private final int registerA;
    private final int registerB;

    public Branch(InstructionType type, int registerA, int registerB, int codeOffset) {
        super(codeOffset, type, IF_EQ, IF_NE, IF_LT, IF_GE, IF_GT, IF_LE);
        this.registerA = registerA;
        this.registerB = registerB;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        byte[] offset = super.serializeBody(methodBuilder);
        return ByteBuffer.allocate(5)
                .put((byte) ((registerA & 0xF) | ((registerB << 4) & 0xF0)))
                .put(offset)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        String operation = (String) type.getGenerateArgs()[0];
        builder.addStatement("int regA = " + readLowNibble())
                .addStatement("int regB = " + readHighNibble())
                .addStatement("int target = " + readInt());

        if (type == IF_EQ || type == IF_NE) {
            builder.beginControlFlow("if(!(regs[regA] instanceof InterpreterNumber))")
                    .beginControlFlow("if(regs[regA] $L regs[regB])", operation)
                    .addStatement("pc = target")
                    .addStatement("continue")
                    .endControlFlow()
                    .endControlFlow()
                    .add("else ");
        }
        builder.beginControlFlow("if(((InterpreterNumber) regs[regA]).intValue() $L ((InterpreterNumber) regs[regB]).intValue())", operation)
                .addStatement("pc = target")
                .addStatement("continue")
                .endControlFlow()
                .addStatement("break");
    }
}
