package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class BranchZero extends OffsetInstruction {
    private final int registerA;

    public BranchZero(InstructionType type, int registerA, int codeOffset) {
        super(codeOffset, type, IF_EQZ, IF_NEZ, IF_LTZ, IF_GEZ, IF_GTZ, IF_LEZ);
        this.registerA = registerA;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        byte[] offset = super.serializeBody(methodBuilder);
        return ByteBuffer.allocate(5).put((byte) registerA)
                .put(offset)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        String operation = (String) type.getGenerateArgs()[0];
        builder.addStatement("int regA = " + readUnsignedByte())
                .addStatement("int target = " + readInt());

        if (type == IF_EQZ || type == IF_NEZ) {
            builder.beginControlFlow("if(!(regs[regA] instanceof InterpreterNumber))")
                    .beginControlFlow("if(regs[regA] $L null)", operation)
                    .addStatement("pc = target")
                    .addStatement("continue")
                    .endControlFlow()
                    .endControlFlow()
                    .add("else ");
        }
        builder.beginControlFlow("if(((InterpreterNumber) regs[regA]).intValue() $L 0)", operation)
                .addStatement("pc = target")
                .addStatement("continue")
                .endControlFlow()
                .addStatement("break");
    }
}
