package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.CHECK_CAST;

public class CheckCast extends Instruction {
    private final int reg;
    private final String type;

    public CheckCast(int reg, String type) {
        super(CHECK_CAST, CHECK_CAST);
        this.reg = reg;
        this.type = type;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(3).putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference(type))
                .put((byte) reg)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("String type = vmoDex.types[$L]", readUnsignedShort())
                // no casting with interpreternumbers, especially important for null-case
                .addStatement("Utils.loadClassByName(type, classLoader).cast(Utils.vmTypeToExternalType(type, regs[$L]))", readUnsignedByte())
                .addStatement("break");
    }
}
