package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import lombok.Getter;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

@Getter
public class Const extends Instruction {
    private final int to;
    private final Object value;

    public Const(InstructionType type, int to, int value) {
        super(type, CONST);
        this.to = to;
        this.value = value;
    }

    public Const(InstructionType type, int to, long value) {
        super(type, CONST_WIDE);
        this.to = to;
        this.value = value;
    }

    public Const(InstructionType type, int to, Object reference) {
        super(type, CONST_STRING, CONST_CLASS);
        this.to = to;
        this.value = reference;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        ByteBuffer buffer = ByteBuffer.allocate(getInstructionType().getSize() - 1);
        buffer.put((byte) to);
        switch (getInstructionType()) {
            case CONST:
                buffer.putInt((Integer) value);
                break;
            case CONST_WIDE:
                buffer.putLong((Long) value);
                break;
            case CONST_STRING:
                buffer.putInt(methodBuilder.getVmoDexBuilder().addStringReference((String) value));
                break;
            case CONST_CLASS:
                buffer.putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference((String) value));
                break;
        }
        return buffer.array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        switch (type) {
            case CONST:
                builder.addStatement("regs[$L] = new InterpreterNumber($L)", readUnsignedByte(), readInt());
                break;
            case CONST_STRING:
                builder.addStatement("regs[$L] = vmoDex.strings[$L]", readUnsignedByte(), readInt());
                break;
            case CONST_CLASS:
                builder.addStatement("regs[$L] = Utils.loadClassByName(vmoDex.types[$L], classLoader)", readUnsignedByte(), readUnsignedShort());
                break;
            case CONST_WIDE:
                builder.addStatement("regs[$L] = new InterpreterNumber($L)", readUnsignedByte(), readLong());
                break;
        }
        builder.addStatement("break");
    }
}
