package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.lang.reflect.Array;
import java.nio.ByteBuffer;
import java.util.List;

import static de.marstra.dvo.obfuscator.insn.InstructionType.FILL_ARRAY_DATA;

public class FillArrayData extends Instruction {
    private final int array;
    private final ArrayPayload payload;

    public FillArrayData(int array, int elementWidth, List<Number> elements) {
        super(FILL_ARRAY_DATA, FILL_ARRAY_DATA);
        this.array = array;
        this.payload = new ArrayPayload(elementWidth, elements);
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(5).put((byte) array)
                .putInt(methodBuilder.addPayload(payload.serialize()))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("Object array = regs[$L]", readUnsignedByte())
                .addStatement("String baseType = Utils.primitiveNameToType(array.getClass().getComponentType().getName())")
                .addStatement("int pos = $L", readInt())
                .addStatement("byte elementWidth = insn[pos++]")
                .addStatement("int numElements = ((insn[pos++] & 0xFF) << 24) | ((insn[pos++] & 0xFF) << 16) | ((insn[pos++] & 0xFF) << 8) | (insn[pos++] & 0xFF)")
                .beginControlFlow("for(int i = 0; i < numElements; i++)")
                .addStatement("long value = 0")
                .beginControlFlow("for(int j = 0; j < elementWidth; j++)")
                // TODO right endianess?
                .addStatement("value = (value << 8) | (insn[pos++] & 0xFF)")
                .endControlFlow()
                .addStatement("Object realValue = Utils.vmTypeToExternalType(baseType, new InterpreterNumber(value))")
                .addStatement("$T.set(array, i, realValue)", Array.class)
                .endControlFlow()
                .addStatement("break");
    }

    private static class ArrayPayload {
        private final int elementWidth;
        private final List<Number> elements;

        public ArrayPayload(int elementWidth, List<Number> elements) {
            this.elementWidth = elementWidth;
            this.elements = elements;
        }

        byte[] serialize() {
            ByteBuffer buffer = ByteBuffer.allocate(5 + elementWidth * elements.size())
                    .put((byte) elementWidth)
                    .putInt(elements.size());

            for (Number ele : elements) {
                switch (elementWidth) {
                    case 1: buffer.put(ele.byteValue()); break;
                    case 2: buffer.putShort(ele.shortValue()); break;
                    case 4: buffer.putInt(ele.intValue()); break;
                    case 8: buffer.putLong(ele.longValue()); break;
                }
            }
            return buffer.array();
        }
    }
}
