package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.lang.reflect.Array;
import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.FILLED_NEW_ARRAY;

public class FilledNewArray extends Instruction {
    private final String type;
    private final int cnt;
    private final int reg0;
    private final int reg1;
    private final int reg2;
    private final int reg3;
    private final int reg4;

    public FilledNewArray(String type, int cnt, int reg0, int reg1, int reg2, int reg3, int reg4) {
        super(FILLED_NEW_ARRAY, FILLED_NEW_ARRAY);
        this.type = type;
        this.cnt = cnt;
        this.reg0 = reg0;
        this.reg1 = reg1;
        this.reg2 = reg2;
        this.reg3 = reg3;
        this.reg4 = reg4;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(5)
                .putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference(type))
                .put((byte) ((cnt & 0xF) | ((reg0 << 4) & 0xF0)))
                .put((byte) ((reg1 & 0xF) | ((reg2 << 4) & 0xF0)))
                .put((byte) ((reg3 & 0xF) | ((reg4 << 4) & 0xF0)))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("$T type = Utils.loadClassByName(vmoDex.types[$L], classLoader).getComponentType()", Class.class, readUnsignedShort())
                .addStatement("String typeName = type.isPrimitive() ? Utils.primitiveNameToType(type.getName()) : \"L\"")
                .addStatement("int length = $L", readLowNibble())
                .addStatement("Object array = $T.newInstance(type, length)", Array.class)
                .addStatement("int[] argRegs = new int[]{$L, $L, $L, $L, $L}", readHighNibble(), readLowNibble(), readHighNibble(), readLowNibble(), readHighNibble())
                .beginControlFlow("for (int i = 0; i < length; i++)")
                .addStatement("Object nativeVal = Utils.vmTypeToExternalType(typeName, regs[argRegs[i]])")
                .addStatement("$T.set(array, i, nativeVal)", Array.class)
                .endControlFlow()
                .addStatement("lastResult = array")
                .addStatement("break");
    }
}

