package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.lang.reflect.Array;
import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.FILLED_NEW_ARRAY_RANGE;

public class FilledNewArrayRange extends Instruction {
    private final String type;
    private final int cnt;
    private final int start;

    public FilledNewArrayRange(String type, int cnt, int start) {
        super(FILLED_NEW_ARRAY_RANGE, FILLED_NEW_ARRAY_RANGE);
        this.type = type;
        this.cnt = cnt;
        this.start = start;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(6).putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference(type))
                .putShort((short) cnt)
                .putShort((short) start)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("$T type = Utils.loadClassByName(vmoDex.types[$L], classLoader).getComponentType()", Class.class, readUnsignedShort())
                .addStatement("String typeName = type.isPrimitive() ? Utils.primitiveNameToType(type.getName()) : \"L\"")
                .addStatement("int length = $L", readUnsignedShort())
                .addStatement("Object array = $T.newInstance(type, length)", Array.class)
                .addStatement("int curReg = $L", readUnsignedShort())
                .beginControlFlow("for (int i = 0; i < length; i++)")
                .addStatement("Object nativeVal = Utils.vmTypeToExternalType(typeName, regs[curReg + i])")
                .addStatement("$T.set(array, i, nativeVal)", Array.class)
                .endControlFlow()
                .addStatement("lastResult = array")
                .addStatement("break");
    }
}

