package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import interpreter.FieldReference;

import java.lang.reflect.Field;
import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.IGET;
import static de.marstra.dvo.obfuscator.insn.InstructionType.IPUT;

public class InstanceLoadStore extends Instruction {
    private final int value;
    private final int object;
    private final FieldReference fieldReference;

    public InstanceLoadStore(InstructionType instructionType, int value, int object, FieldReference fieldReference) {
        super(instructionType, IGET, IPUT);
        this.value = value;
        this.object = object;
        this.fieldReference = fieldReference;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        int fref = methodBuilder.getVmoDexBuilder().addFieldReference(fieldReference);
        return ByteBuffer.allocate(4).put((byte) ((value & 0xF) | ((object << 4) & 0xF0)))
                .put((byte) ((fref >> 16) & 0xFF))
                .put((byte) ((fref >> 8) & 0xFF))
                .put((byte) (fref & 0xFF))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        boolean set = (boolean) type.getGenerateArgs()[0];
        builder.addStatement("int regNum = $L", readLowNibble())
                .addStatement("Object object = regs[$L]", readHighNibble())
                .addStatement("FieldReference fr = vmoDex.frefs[(((insn[pc++] & 0xFF) << 16) | ((insn[pc++] & 0xFF) << 8) | (insn[pc++] & 0xFF))]")
                .addStatement("$T targetClass = Utils.loadClassByName(fr.definingClass, classLoader)", Class.class)
                .addStatement("$T field = Utils.getField(targetClass, fr.name)", Field.class)
                .addStatement("field.setAccessible(true)");
        if (set) {
            // NOTE also for object type important to convert InterpreterNumber(0) -> null
            builder.addStatement("field.set(object, Utils.vmTypeToExternalType(fr.type, regs[regNum]))");
        } else {
            builder.addStatement("Object nativeVal = field.get(object)");
            builder.addStatement("regs[regNum] = Utils.externalTypeToVmType(fr.type, nativeVal)");
        }
        builder.addStatement("break");
    }
}
