package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.INSTANCE_OF;

public class InstanceOf extends Instruction {
    private final int target;
    private final int ref;
    private final String type;

    public InstanceOf(int target, int ref, String type) {
        super(INSTANCE_OF, INSTANCE_OF);
        this.target = target;
        this.ref = ref;
        this.type = type;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(3).put((byte) ((target & 0xF) | ((ref << 4) & 0xF0)))
                .putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference(type))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("int reg1 = " + readLowNibble())
                .addStatement("int reg2 = " + readHighNibble())
                .addStatement("regs[reg1] = Utils.loadClassByName(vmoDex.types[$L], classLoader).isInstance(regs[reg2]) ? InterpreterNumber.ONE : InterpreterNumber.ZERO", readUnsignedShort())
                .addStatement("break");
    }
}
