package de.marstra.dvo.obfuscator.insn;

import de.marstra.dvo.obfuscator.VMODexCompiler;
import lombok.Getter;

import java.util.Arrays;

@Getter
public abstract class Instruction {
    private final InstructionType instructionType;

    protected Instruction(InstructionType instructionType, InstructionType... allowedTypes) {
        this.instructionType = instructionType;
        if (Arrays.stream(allowedTypes).noneMatch(instructionType::equals))
            throw new IllegalArgumentException("instruction is of invalid type");
    }

    public byte[] serialize(VMODexCompiler.MethodBuilder methodBuilder) {
        byte[] body = serializeBody(methodBuilder);
        byte[] serialized = new byte[body.length + 1];
        serialized[0] = methodBuilder.getVmoDexBuilder().getByteCodeMapping().toVmOpcode(instructionType);
        System.arraycopy(body, 0, serialized, 1, body.length);

        if (serialized.length != instructionType.getSize())
            throw new IllegalStateException("serialization of " + instructionType + " has wrong size");

        return serialized;
    }

    abstract byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder);

    static String readLowNibble() {
        return "((byte) (insn[pc] & 0xF))";
    }

    static String readHighNibble() {
        return "((byte) ((insn[pc++] >> 4) & 0xF))";
    }

    static String readUnsignedByte() {
        return "((short) (insn[pc++] & 0xFF))";
    }

    static String readUnsignedShort() {
        return "((int) ((insn[pc++] & 0xFF) << 8 | (insn[pc++] & 0xFF)))";
    }

    static String readShort() {
        return "((short) (insn[pc++] << 8 | (insn[pc++] & 0xFF)))";
    }

    static String readInt() {
        // avoid sign extension when converting from byte to int by using "& 0xFF" to convert to int
        return "(((insn[pc++] & 0xFF) << 24) | ((insn[pc++] & 0xFF) << 16) | ((insn[pc++] & 0xFF) << 8) | (insn[pc++] & 0xFF))";
    }

    static String readLong() {
        return "((((long) " + readInt() + ") << 32L) | ((long) " + readInt() + " & 0xFFFFFFFFL))";
    }
}