package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import interpreter.MethodReference;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class Invoke extends InvokeInstruction {
    private final MethodReference methodReference;
    private final int reg0;
    private final int reg1;
    private final int reg2;
    private final int reg3;
    private final int reg4;

    public Invoke(InstructionType instructionType, MethodReference methodReference, int reg0, int reg1, int reg2, int reg3, int reg4) {
        super(instructionType, INVOKE_VIRTUAL, INVOKE_DIRECT, INVOKE_STATIC, INVOKE_INTERFACE);
        this.methodReference = methodReference;
        this.reg0 = reg0;
        this.reg1 = reg1;
        this.reg2 = reg2;
        this.reg3 = reg3;
        this.reg4 = reg4;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        int mref = methodBuilder.getVmoDexBuilder().addMethodReference(methodReference);
        return ByteBuffer.allocate(5)
                .put((byte) ((mref >> 12) & 0xFF))
                .put((byte) ((mref >> 4) & 0xFF))
                .put((byte) ((mref & 0xF) | ((reg0 << 4) & 0xF0)))
                .put((byte) ((reg1 & 0xF) | ((reg2 << 4) & 0xF0)))
                .put((byte) ((reg3 & 0xF) | ((reg4 << 4) & 0xF0)))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        generateRetrieveParameterTypes(builder, type);

        builder.addStatement("Object[] margs = new Object[pTypes.length]")
                .addStatement("int[] argRegs = new int[]{$L, $L, $L, $L, $L}",
                        readHighNibble(),
                        readLowNibble(),
                        readHighNibble(),
                        readLowNibble(),
                        readHighNibble())
                .addStatement("final int thisReg = argRegs[0]")
                .addStatement("int curReg = " + (type == INVOKE_STATIC ? "0" : "1"))
                .beginControlFlow("for (int i = 0; i < margs.length; i++)")
                .addStatement("margs[i] = Utils.vmTypeToExternalType(mr.parameterTypes[i], regs[argRegs[curReg]])")
                .addStatement("curReg++")
                .addStatement("if (Utils.isWideType(mr.parameterTypes[i])) curReg++")
                .endControlFlow();

        generateExecuteInvoce(builder, type);
    }
}

