package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

abstract class InvokeInstruction extends Instruction {
    InvokeInstruction(InstructionType instructionType, InstructionType... allowedTypes) {
        super(instructionType, allowedTypes);
    }

    private static CodeBlock unpackInvocationTargetException(String code) {
        CodeBlock.Builder builder = CodeBlock.builder();
        builder.beginControlFlow("try")
                .addStatement(code)
                .nextControlFlow("catch ($T ite)", InvocationTargetException.class)
                .addStatement("throw ite.getCause()")
                .endControlFlow();
        return builder.build();
    }

    static void generateRetrieveParameterTypes(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("MethodReference mr = vmoDex.mrefs[(((insn[pc++] & 0xFF) << 12) | ((insn[pc++] & 0xFF) << 4) | (insn[pc] & 0xF))]")
                .addStatement("Class defClass = Utils.loadClassByName(mr.definingClass, classLoader)")
                .addStatement("Class rType = Utils.loadClassByName(mr.returnType, classLoader)")
                .addStatement("Class[] pTypes = new Class[mr.parameterTypes.length]")
                .beginControlFlow("for (int i = 0; i < pTypes.length; i++)")
                .addStatement("pTypes[i] = Utils.loadClassByName(mr.parameterTypes[i], classLoader)")
                .endControlFlow();
    }

    static void generateExecuteInvoce(CodeBlock.Builder builder, InstructionType type) {
        if (type == INVOKE_STATIC || type == INVOKE_STATIC_RANGE) {
            builder.addStatement("$T m = Utils.getMethod(defClass, mr.name, pTypes)", Method.class)
                    .add(unpackInvocationTargetException("lastResult = m.invoke(null, margs)"))
                    .addStatement("lastResult = Utils.externalTypeToVmType(mr.returnType,lastResult)");
        } else if (type == INVOKE_DIRECT || type == INVOKE_DIRECT_RANGE) {
            // handle constructors
            builder.addStatement("Object thisArg = regs[thisReg]")
                    .beginControlFlow("if(mr.name.equals(\"<init>\"))")
                    .addStatement("$T constructor = Utils.getConstructor(((UninitializedInstance) thisArg).clazz, pTypes)", Constructor.class)
                    .addStatement("Object newInstance = null")
                    .add(unpackInvocationTargetException("newInstance = constructor.newInstance(margs)"))
                    .addStatement("((UninitializedInstance) thisArg).init(regs, newInstance)")
                    .nextControlFlow("else")
                    .addStatement("$T m = Utils.getMethod(defClass, mr.name, pTypes)", Method.class)
                    .add(unpackInvocationTargetException("lastResult = m.invoke(thisArg, margs)"))
                    .addStatement("lastResult = Utils.externalTypeToVmType(mr.returnType,lastResult)")
                    .endControlFlow();
        } else {
            builder.addStatement("Object thisArg = regs[thisReg]")
                    .addStatement("$T m = Utils.getMethod(defClass, mr.name, pTypes)", Method.class)
                    .add(unpackInvocationTargetException("lastResult = m.invoke(thisArg, margs)"))
                    .addStatement("lastResult = Utils.externalTypeToVmType(mr.returnType,lastResult)");
        }
        builder.addStatement("break");
    }
}
