package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import interpreter.MethodReference;

import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

public class InvokeRange extends InvokeInstruction {
    private final MethodReference methodReference;
    private final int start;

    public InvokeRange(InstructionType instructionType, MethodReference methodReference, int start) {
        super(instructionType, INVOKE_VIRTUAL_RANGE, INVOKE_DIRECT_RANGE, INVOKE_STATIC_RANGE, INVOKE_INTERFACE_RANGE);
        this.methodReference = methodReference;
        this.start = start;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        int mref = methodBuilder.getVmoDexBuilder().addMethodReference(methodReference);
        return ByteBuffer.allocate(5)
                .put((byte) ((mref >> 12) & 0xFF))
                .put((byte) ((mref >> 4) & 0xFF))
                .put((byte) (mref & 0xF))
                .putShort((short) start)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        generateRetrieveParameterTypes(builder, type);

        builder.addStatement("Object[] margs = new Object[pTypes.length]")
                .addStatement("pc++")
                .addStatement("final int startReg = $L", readUnsignedShort())
                .addStatement("final int thisReg = startReg")
                .addStatement("int curReg = startReg" + (type == INVOKE_STATIC_RANGE ? "" : " + 1"))
                .beginControlFlow("for (int i = 0; i < margs.length; i++)")
                .addStatement("margs[i] = Utils.vmTypeToExternalType(mr.parameterTypes[i], regs[curReg])")
                .addStatement("curReg++")
                .addStatement("if (Utils.isWideType(mr.parameterTypes[i])) curReg++")
                .endControlFlow();

        generateExecuteInvoce(builder, type);
    }
}

