package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import lombok.Getter;

import java.nio.ByteBuffer;
import java.util.Optional;

import static de.marstra.dvo.obfuscator.insn.InstructionType.*;

@Getter
public class Move extends Instruction {
    private final int to;
    private final Optional<Integer> from;

    public Move(InstructionType type, int to, int from) {
        super(type, MOVE, MOVE_OBJECT);
        this.to = to;
        this.from = Optional.of(from);
    }

    public Move(InstructionType type, int to) {
        super(type, MOVE_EXCEPTION, MOVE_RESULT);
        this.to = to;
        this.from = Optional.empty();
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        ByteBuffer buffer = ByteBuffer.allocate(getInstructionType().getSize() - 1);
        from.ifPresent(integer -> buffer.putShort((short) integer.intValue()));
        buffer.putShort((short) to);
        return buffer.array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        if (type.getGenerateArgs().length > 0) {
            builder.addStatement("regs[$L] = " + type.getGenerateArgs()[0], readUnsignedShort())
                    .addStatement("break");
            return;
        }

        builder.addStatement("int dstReg = " + readUnsignedShort())
                .addStatement("int srcReg = " + readUnsignedShort());

        // TODO remove this flaw by combining new_instance and call to <init>
        if (type == MOVE_OBJECT) {
            // Register fdest now also refers to this UninitializedInstance,
            // therefore a reference is added to the UninitializedInstance,
            // so when <init> is called this reference will also be updated to the real instance
            builder.beginControlFlow("if (regs[srcReg] instanceof UninitializedInstance)")
                    .addStatement("((UninitializedInstance) regs[srcReg]).addReference(dstReg)")
                    .endControlFlow();
        }
        builder.addStatement("regs[dstReg] = regs[srcReg]")
                .addStatement("break");
    }
}
