package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.lang.reflect.Array;
import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.NEW_ARRAY;

public class NewArray extends Instruction {
    private final int dst;
    private final int size;
    private final String type;

    public NewArray(int dst, int size, String type) {
        super(NEW_ARRAY, NEW_ARRAY);
        this.dst = dst;
        this.size = size;
        this.type = type;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(4)
                .putShort((short) methodBuilder.getVmoDexBuilder().addTypeReference(type))
                .put((byte) dst)
                .put((byte) size)
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("$T type = Utils.loadClassByName(vmoDex.types[$L], classLoader).getComponentType()", Class.class, readUnsignedShort())
                .addStatement("regs[$L] = $T.newInstance(type, ((InterpreterNumber) regs[$L]).intValue())", readUnsignedByte(), Array.class, readUnsignedByte())
                .addStatement("break");

    }
}
