package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;

import java.util.Optional;

import static de.marstra.dvo.obfuscator.insn.InstructionType.RETURN;
import static de.marstra.dvo.obfuscator.insn.InstructionType.RETURN_VOID;

public class Return extends Instruction {
    private final Optional<Integer> op;

    public Return(InstructionType type) {
        super(type, RETURN_VOID);
        this.op = Optional.empty();
    }

    public Return(InstructionType type, int op) {
        super(type, RETURN);
        this.op = Optional.of(op);
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        if(op.isPresent())
            return new byte[]{(byte) op.get().intValue()};
        return new byte[0];
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        if (type == RETURN_VOID)
            builder.addStatement("return null");
        else
            builder.addStatement("return Utils.vmTypeToExternalType(method.returnType, regs[$L])", readUnsignedByte());
    }
}
