package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import interpreter.FieldReference;

import java.lang.reflect.Field;
import java.nio.ByteBuffer;

import static de.marstra.dvo.obfuscator.insn.InstructionType.SGET;
import static de.marstra.dvo.obfuscator.insn.InstructionType.SPUT;

public class StaticLoadStore extends Instruction {
    private final int value;
    private final FieldReference fieldReference;

    public StaticLoadStore(InstructionType instructionType, int value, FieldReference fieldReference) {
        super(instructionType, SGET, SPUT);
        this.value = value;
        this.fieldReference = fieldReference;
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        int fref = methodBuilder.getVmoDexBuilder().addFieldReference(this.fieldReference);
        return ByteBuffer.allocate(4).put((byte) value)
                .put((byte) ((fref >> 16) & 0xFF))
                .put((byte) ((fref >> 8) & 0xFF))
                .put((byte) (fref & 0xFF))
                .array();
    }

    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        boolean set = (boolean) type.getGenerateArgs()[0];
        builder.addStatement("int regNum = $L", readUnsignedByte())
                .addStatement("FieldReference fr = vmoDex.frefs[(((insn[pc++] & 0xFF) << 16) | ((insn[pc++] & 0xFF) << 8) | (insn[pc++] & 0xFF))]")
                .addStatement("$T targetClass = Utils.loadClassByName(fr.definingClass, classLoader)", Class.class)
                .addStatement("$T field = Utils.getField(targetClass, fr.name)", Field.class)
                .addStatement("field.setAccessible(true)");
        if (set) {
            // NOTE also for object type important to convert InterpreterNumber(0) -> null
            builder.addStatement("field.set(null, Utils.vmTypeToExternalType(fr.type, regs[regNum]))");
        } else {
            builder.addStatement("Object nativeVal = field.get(null)");
            builder.addStatement("regs[regNum] = Utils.externalTypeToVmType(fr.type, nativeVal)");
        }
        builder.addStatement("break");
    }
}
