package de.marstra.dvo.obfuscator.insn;

import com.squareup.javapoet.CodeBlock;
import de.marstra.dvo.obfuscator.VMODexCompiler;
import org.jf.dexlib2.iface.instruction.SwitchElement;

import java.nio.ByteBuffer;
import java.util.List;

import static de.marstra.dvo.obfuscator.insn.InstructionType.SWITCH;

public class Switch extends Instruction {
    private final int reg;
    private final SwitchPayload payload;

    public Switch(int reg, List<? extends SwitchElement> switchElements) {
        super(SWITCH, SWITCH);
        this.reg = reg;
        this.payload = new SwitchPayload(this, switchElements);
    }

    @Override
    byte[] serializeBody(VMODexCompiler.MethodBuilder methodBuilder) {
        return ByteBuffer.allocate(5).put((byte) reg)
                .putInt(methodBuilder.addPayload(payload.serialize(methodBuilder)))
                .array();
    }

    // TODO implement in a faster way by using a lookup table
    public static void generateCase(CodeBlock.Builder builder, InstructionType type) {
        builder.addStatement("InterpreterNumber regToTest = (InterpreterNumber) regs[$L]", readUnsignedByte())
                .addStatement("int pos = $L", readInt())
                .addStatement("short size = ((short) (insn[pos++] << 8 | (insn[pos++] & 0xFF)))")
                .beginControlFlow("for (int i = 0; i < size; i++)")
                .addStatement("int key = ((insn[pos++] & 0xFF) << 24) | ((insn[pos++] & 0xFF) << 16) | ((insn[pos++] & 0xFF) << 8) | (insn[pos++] & 0xFF)")
                .addStatement("int target = ((insn[pos++] & 0xFF) << 24) | ((insn[pos++] & 0xFF) << 16) | ((insn[pos++] & 0xFF) << 8) | (insn[pos++] & 0xFF)")
                .beginControlFlow("if (key == regToTest.intValue())")
                .addStatement("pc = target")
                .addStatement("continue interpreterLoop")
                .endControlFlow()
                .endControlFlow()
                .addStatement("break");
    }

    private static class SwitchPayload {
        private final Switch switchInstruction;
        private final List<? extends SwitchElement> switchElements;

        public SwitchPayload(Switch switchInstruction, List<? extends SwitchElement> switchElements) {
            this.switchInstruction = switchInstruction;
            this.switchElements = switchElements;
        }

        byte[] serialize(VMODexCompiler.MethodBuilder methodBuilder) {
            ByteBuffer buffer = ByteBuffer.allocate(2 + switchElements.size() * 8);
            buffer.putShort((short) switchElements.size());
            for (SwitchElement switchElement : switchElements) {
                buffer.putInt(switchElement.getKey());
                buffer.putInt(methodBuilder.translateCodeOffset(switchInstruction, switchElement.getOffset()));
            }
            return buffer.array();
        }
    }
}
