package de.marstra.dvo.obfuscator.util;

import lombok.Getter;

import java.util.HashMap;
import java.util.Map;

class WrapperType {
    private char sign;
    private String name;
    private String wrapperClass;
    @Getter
    private GenericMethodReference wrap;
    @Getter
    private GenericMethodReference unwrap;

    public WrapperType(char sign, String name, String wrapperClass) {
        this.sign = sign;
        this.name = name;
        this.wrapperClass = wrapperClass;
        this.wrap = new GenericMethodReference(wrapperClass, "valueOf", wrapperClass, String.valueOf(sign));
        this.unwrap = new GenericMethodReference(wrapperClass, name + "Value", String.valueOf(sign), new String[0]);
    }
}

public class PrimitiveWrapperUtils {
    private static final Map<Character, WrapperType> typeMap = new HashMap<>();

    static {
        typeMap.put('Z', new WrapperType('Z', "boolean", "Ljava/lang/Boolean;"));
        typeMap.put('B', new WrapperType('B', "byte", "Ljava/lang/Byte;"));
        typeMap.put('S', new WrapperType('S', "short", "Ljava/lang/Short;"));
        typeMap.put('C', new WrapperType('C', "char", "Ljava/lang/Character;"));
        typeMap.put('I', new WrapperType('I', "int", "Ljava/lang/Integer;"));
        typeMap.put('J', new WrapperType('J', "long", "Ljava/lang/Long;"));
        typeMap.put('F', new WrapperType('F', "float", "Ljava/lang/Float;"));
        typeMap.put('D', new WrapperType('D', "double", "Ljava/lang/Double;"));
    }

    public static GenericMethodReference getWrapMethod(String type) {
        WrapperType wrapperType = typeMap.get(type.charAt(0));
        if (wrapperType != null)
            return wrapperType.getWrap();
        return null;
    }

    public static GenericMethodReference getUnwrapMethod(String type) {
        WrapperType wrapperType = typeMap.get(type.charAt(0));
        if (wrapperType != null)
            return wrapperType.getUnwrap();
        return null;
    }
}
