package interpreter;

import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;
import java.util.Objects;

public final class FieldReference {
    public final String definingClass;
    public final String name;
    public final String type;

    public FieldReference(String definingClass, String name, String type) {
        this.definingClass = definingClass;
        this.name = name;
        this.type = type;
    }

    public byte[] serialize() {
        ByteArrayOutputStream baout = new ByteArrayOutputStream();
        Utils.serializeString(baout, definingClass);
        Utils.serializeString(baout, name);
        Utils.serializeString(baout, type);
        return baout.toByteArray();
    }

    public static FieldReference deserialize(ByteBuffer byteBuf) {
        return new FieldReference(
                Utils.deserializeString(byteBuf),
                Utils.deserializeString(byteBuf),
                Utils.deserializeString(byteBuf)
        );
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        FieldReference that = (FieldReference) o;
        return Objects.equals(definingClass, that.definingClass) &&
                Objects.equals(name, that.name) &&
                Objects.equals(type, that.type);
    }

    @Override
    public int hashCode() {
        return Objects.hash(definingClass, name, type);
    }
}
