package interpreter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Objects;

public final class MethodReference {
    public final String definingClass;
    public final String name;
    public final String[] parameterTypes;
    public final String returnType;

    public MethodReference(String definingClass, String name, String[] parameterTypes, String returnType) {
        this.definingClass = definingClass;
        this.name = name;
        this.parameterTypes = parameterTypes;
        this.returnType = returnType;
    }

    public byte[] serialize() {
        ByteArrayOutputStream baout = new ByteArrayOutputStream();
        Utils.serializeString(baout, definingClass);
        Utils.serializeString(baout, name);
        try {
            baout.write(Utils.intToBytes(parameterTypes.length));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        for (String parameterType : parameterTypes) {
            Utils.serializeString(baout, parameterType);
        }
        Utils.serializeString(baout, returnType);
        return baout.toByteArray();
    }

    public static MethodReference deserialize(ByteBuffer byteBuf) {
        String definingClass = Utils.deserializeString(byteBuf);
        String name = Utils.deserializeString(byteBuf);
        String[] parameterTypes = new String[byteBuf.getInt()];
        for (int i = 0; i < parameterTypes.length; i++) {
            parameterTypes[i] = Utils.deserializeString(byteBuf);
        }
        String returnType = Utils.deserializeString(byteBuf);
        return new MethodReference(definingClass, name, parameterTypes, returnType);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MethodReference that = (MethodReference) o;
        return Objects.equals(definingClass, that.definingClass) &&
                Objects.equals(name, that.name) &&
                Arrays.equals(parameterTypes, that.parameterTypes) &&
                Objects.equals(returnType, that.returnType);
    }

    @Override
    public int hashCode() {
        int result = Objects.hash(definingClass, name, returnType);
        result = 31 * result + Arrays.hashCode(parameterTypes);
        return result;
    }
}
