package interpreter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

public final class TryBlock {
    public final int start;
    public final int end;
    public final ExceptionHandler[] exceptionHandlers;

    public TryBlock(int start, int end, ExceptionHandler[] exceptionHandlers) {
        this.start = start;
        this.end = end;
        this.exceptionHandlers = exceptionHandlers;
    }

    public static TryBlock deserialize(ByteBuffer buf) {
        int start = buf.getInt();
        int end = buf.getInt();
        ExceptionHandler[] handlers = new ExceptionHandler[buf.getInt()];
        for (int i = 0; i < handlers.length; i++) {
            handlers[i] = ExceptionHandler.deserialize(buf);
        }
        return new TryBlock(start, end, handlers);
    }

    public byte[] serialize() {
        ByteArrayOutputStream baout = new ByteArrayOutputStream();
        try {
            baout.write(Utils.intToBytes(start));
            baout.write(Utils.intToBytes(end));
            baout.write(Utils.intToBytes(exceptionHandlers.length));
            for (ExceptionHandler handler : exceptionHandlers) {
                baout.write(handler.serialize());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return baout.toByteArray();
    }
}
