package interpreter;

public final class InterpreterNumber {
    private final long value;

    public static final InterpreterNumber ZERO = new InterpreterNumber(0);
    public static final InterpreterNumber ONE = new InterpreterNumber(1);
    public static final InterpreterNumber MINUS_ONE = new InterpreterNumber(-1);

    public InterpreterNumber(boolean value) {
        this.value = value ? 1 : 0;
    }

    public InterpreterNumber(byte value) {
        this.value = value;
    }

    public InterpreterNumber(char value) {
        this.value = value;
    }

    public InterpreterNumber(short value) {
        this.value = value;
    }

    public InterpreterNumber(int value) {
        this.value = value;
    }

    public InterpreterNumber(long value) {
        this.value = value;
    }

    public InterpreterNumber(float value) {
        this.value = Float.floatToIntBits(value);
    }

    public InterpreterNumber(double value) {
        this.value = Double.doubleToLongBits(value);
    }

    public boolean booleanValue() {
        return value != 0;
    }

    public byte byteValue() {
        return (byte) value;
    }

    public char charValue() {
        return (char) value;
    }

    public short shortValue() {
        return (short) value;
    }

    public int intValue() {
        return (int) value;
    }

    public long longValue() {
        return value;
    }

    public float floatValue() {
        return Float.intBitsToFloat((int) value);
    }

    public double doubleValue() {
        return Double.longBitsToDouble(value);
    }
}
