package interpreter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

public final class Method {
    public final String name;
    public final String definingClass;
    public final int numRegs;
    public final byte[] instructions;
    public final String returnType;
    // contains type of this for non static methods
    public final String[] paramTypes;
    public final TryBlock[] tryBlocks;

    public Method(String name, String definingClass, int numRegs, byte[] instructions, String returnType, String[] paramTypes, TryBlock[] tryBlocks) {
        this.name = name;
        this.definingClass = definingClass;
        this.numRegs = numRegs;
        this.instructions = instructions;
        this.returnType = returnType;
        this.paramTypes = paramTypes;
        this.tryBlocks = tryBlocks;
    }

    public static Method deserialize(ByteBuffer buf) {
        String name = Utils.deserializeString(buf);
        String definingClass = Utils.deserializeString(buf);
        int numRegs = buf.getInt();
        String returnType = Utils.deserializeString(buf);

        String[] paramTypes = new String[buf.getInt()];
        for (int i = 0; i < paramTypes.length; i++) {
            paramTypes[i] = Utils.deserializeString(buf);
        }

        byte[] instructions = new byte[buf.getInt()];
        buf.get(instructions);

        TryBlock[] tryBlocks = new TryBlock[buf.getInt()];
        for (int i = 0; i < tryBlocks.length; i++) {
            tryBlocks[i] = TryBlock.deserialize(buf);
        }

        return new Method(name, definingClass, numRegs, instructions, returnType, paramTypes, tryBlocks);
    }

    public byte[] serialize() {
        ByteArrayOutputStream baout = new ByteArrayOutputStream();
        try {
            Utils.serializeString(baout, name);
            Utils.serializeString(baout, definingClass);
            baout.write(Utils.intToBytes(numRegs));
            Utils.serializeString(baout, returnType);

            baout.write(Utils.intToBytes(paramTypes.length));
            for (String paramType : paramTypes) {
                Utils.serializeString(baout, paramType);
            }

            baout.write(Utils.intToBytes(instructions.length));
            baout.write(instructions);

            baout.write(Utils.intToBytes(tryBlocks.length));
            for (TryBlock tryBlock : tryBlocks) {
                baout.write(tryBlock.serialize());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        return baout.toByteArray();
    }
}
