package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

import java.util.Random;

public class ArithmeticTest {
    @Test
    public void testAdd() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    private static int add(int a, int b) {\n" +
                "        return a + b;\n" +
                "    }" +
                "    public static Object test(Object[] args){\n" +
                "        return add((Integer) args[0], (Integer) args[1]);\n" +
                "    }\n" +
                "}";

        new InterpreterTestCase(code, "LTest;->add(II)I")
                .addInputSample(new Random().nextInt(), new Random().nextInt())
                .addInputSample(new Random().nextInt(), new Random().nextInt())
                .run();
    }

    @Test
    public void testUnOps() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double test(Long val) {\n" +
                "        double checksum = 0;\n" +
                "\n" +
                "        checksum += -val.intValue();\n" +
                "        checksum += -((short) val.intValue());\n" +
                // NOTE might create xor -1, instead of neg opcode because some dalvik-vms might not support neg
                // because old android compilers never emitted neg
                "        checksum += ~val.intValue();\n" +
                "        checksum += ~((short) val.intValue());\n" +
                "        checksum += -val;\n" +
                "        checksum += ~val;\n" +
                "        checksum += -val.floatValue();\n" +
                "        checksum += -val.doubleValue();\n" +
                "\n" +
                "        checksum += (long) val.intValue();\n" +
                "        checksum += (float) val.intValue();\n" +
                "        checksum += (double) val.intValue();\n" +
                "\n" +
                "        checksum += (int) val.longValue();\n" +
                "        checksum += (float) val.longValue();\n" +
                "        checksum += (double) val.longValue();\n" +
                "\n" +
                "        checksum += (int) val.floatValue();\n" +
                "        checksum += (long) val.floatValue();\n" +
                "        checksum += (double) val.floatValue();\n" +
                "\n" +
                "        checksum += (int) val.doubleValue();\n" +
                "        checksum += (long) val.doubleValue();\n" +
                "        checksum += (float) val.doubleValue();\n" +
                "\n" +
                "        checksum += (byte) val.intValue();\n" +
                "        checksum += (char) val.intValue();\n" +
                "        checksum += (short) val.intValue();\n" +
                "\n" +
                "        return checksum;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Long) args[0]);\n" +
                "    }\n" +
                "}";

        new InterpreterTestCase(code, "LTest;->test(Ljava/lang/Long;)D")
                .addInputSample(-Long.valueOf(Long.MAX_VALUE))
                .addInputSample(Long.valueOf(Long.MIN_VALUE))
                .addInputSample(new Random().nextLong())
                .addInputSample(new Random().nextLong())
                .run();
    }

    @Test
    public void testLit8Ops() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int calc(int val) {\n" +
                "        int res = val + 8;\n" +
                "        res = res + Byte.MAX_VALUE;\n" +
                "        res = res - Byte.MIN_VALUE;\n" +
                "        res = res - 4;\n" +
                "        res = res % 97;\n" +
                "        res = res ^ 0xFF;\n" +
                "        res = res & 0xF;\n" +
                "        res = res * 20;\n" +
                "        res = res | 0x7;\n" +
                "        res = res << 17;\n" +
                "        res = res >> 17;\n" +
                "        res = res << 20;\n" +
                "        res = res >>> 20;\n" +
                "        res = res / 7;\n" +
                "        return res;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return calc((Integer) args[0]);\n" +
                "    }\n" +
                "}";

        new InterpreterTestCase(code, "LTest;->calc(I)I")
                .addInputSample(Integer.MAX_VALUE)
                .addInputSample(Integer.MIN_VALUE)
                .addInputSample(0)
                .addInputSample(new Random().nextInt())
                .addInputSample(new Random().nextInt())
                .run();
    }

    @Test
    public void testCmpOps() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean calc() {\n" +
                "        float fnan = Float.NaN;\n" +
                "        double dnan = Double.NaN;\n" +
                "        float f = Float.MIN_VALUE;\n" +
                "        double d = Double.MAX_VALUE;\n" +
                "        long l = Long.MIN_VALUE;\n" +
                "        boolean success = true;\n" +
                "        success &= !(fnan < 42.73f);\n" +
                "        success &= !(fnan == Float.NaN);\n" +
                "        success &= !(dnan < 5.123);\n" +
                "        success &= !(dnan == Double.NaN);\n" +
                "        success &= (f < 5e3f);\n" +
                "        success &= (f == Float.MIN_VALUE);\n" +
                "        success &= (d < 5e7f);\n" +
                "        success &= (d == Double.MIN_VALUE);\n" +
                "        success &= (l < 9123L);\n" +
                "        success &= (l == Long.MIN_VALUE);\n" +
                "        return success;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return calc();\n" +
                "    }\n" +
                "}";

        new InterpreterTestCase(code, "LTest;->calc()Z").run();
    }

    // TODO testcases for litops16, binops, binop2addr
}
