package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

import java.util.Random;

import static org.junit.Assert.assertArrayEquals;

public class ArrayTest {
    @Test
    public void createArray() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int[] test(int len) {\n" +
                "        return new int[len];\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)[I")
                .addInputSample(0)
                .addInputSample(3)
                .addInputSample(17)
                .run();
    }

    @Test
    public void createMultipleFilledArray() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Object[] test() {\n" +
                "        byte[]   a = new byte[]{1,2,3};\n" +
                "        char[]   b = new char[]{'a','b'};\n" +
                "        short[]  c = new short[]{4,5,6,7,8};\n" +
                "        int[]    d = new int[]{9};\n" +
                "        long[]   e = new long[]{10,12};\n" +
                "        float[]  f = new float[]{13.3f,14.444f, 15.55f};\n" +
                "        double[] g = new double[]{16.6,17.7};\n" +
                "        boolean[] h = new boolean[]{false, true, false};\n" +
                "        return new Object[]{a,b,c,d,e,f,g,h};\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[Ljava/lang/Object;").run();
    }

    @Test
    public void arrayLength() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test(int len) {\n" +
                "        return new boolean[len].length + new Object[len].length;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I")
                .addInputSample(0)
                .addInputSample(13)
                .addInputSample(42)
                .run();
    }

    @Test
    public void createFilledArray() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int[] test() {\n" +
                "        return new int[]{1,2,3};\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[I").run();
    }

    @Test
    public void intAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int[] test() {\n" +
                "        int[] array = new int[]{1,2,3};\n" +
                "        array[0] += array[2];\n" +
                "        array[1] *= 3;\n" +
                "        return array;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[I").run();
    }

    @Test
    public void booleanMultiDimAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        boolean[][] h = new boolean[][]{\n" +
                "           {true, false},\n" +
                "           {false, true}\n" +
                "        };\n" +
                "        return !h[1][0];\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    @Test
    public void floatAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static float[] test() {\n" +
                "        float[] array = new float[]{1.3f,2.713f,3.1235f};\n" +
                "        array[0] += array[1];\n" +
                "        array[0] += array[2];\n" +
                "        array[1] *= array[2];\n" +
                "        return array;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[F").run();
    }

    @Test
    public void longAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long[] test() {\n" +
                "        long[] array = new long[]{Long.MAX_VALUE,0x4242424242424242L,1L};\n" +
                "        array[0] += array[1];\n" +
                "        array[0] += array[2];\n" +
                "        array[1] *= array[2];\n" +
                "        return array;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[J").run();
    }

    @Test
    public void doubleAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double[] test() {\n" +
                "        double[] array = new double[]{Double.MAX_VALUE, 1.35,Double.MIN_VALUE};\n" +
                "        array[0] += array[1];\n" +
                "        array[0] += array[2];\n" +
                "        array[1] *= array[2];\n" +
                "        return array;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()[D").run();
    }

    @Test
    public void objectAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Object test() {\n" +
                "        Object[] a = new Object[5];\n" +
                "        a[3] = \"Test\";\n" +
                "        a[0] = a[3];\n" +
                "        return a[0];\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }
}
