package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class ConstTest {
    @Test
    public void testConstString() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static String test() {\n" +
                "        return \"Hello World!\";\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/String;").run();
    }

    @Test
    public void testConstClass() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Class test() {\n" +
                "        return Integer.class;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Class;").run();
    }

    @Test
    public void testConstClass2() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Class test() {\n" +
                "        return long[][][].class;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Class;").run();
    }

    @Test
    public void testConstClass3() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Class test() {\n" +
                "        return Test.class;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test().toString().replaceAll(\"(orig|patched)\", \"\");\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Class;").run();
    }

    @Test
    public void testConstClass4() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Class test() {\n" +
                "        return Test[][].class;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test().toString().replaceAll(\"(orig|patched)\", \"\");\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Class;").run();
    }

    @Test
    public void testConst4() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test() {\n" +
                "        return 4;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void testConst16() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test() {\n" +
                "        return Byte.MAX_VALUE;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void testConst() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test() {\n" +
                "        return Short.MAX_VALUE + 1;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void testConstWide16() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long test() {\n" +
                "        return Byte.MAX_VALUE;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()J").run();
    }

    @Test
    public void testConstWide32() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long test() {\n" +
                "        return Short.MAX_VALUE + 1;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()J").run();
    }

    @Test
    public void testConstWide() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long test() {\n" +
                "        return Long.MAX_VALUE;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()J").run();
    }

    @Test
    public void testConstHigh() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static float test() {\n" +
                "        return 1.75f;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()F").run();
    }


    @Test
    public void testConstWideHigh() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double test() {\n" +
                "        return 1.75;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()D").run();
    }
}
