package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

public class ObjectOrientationTest {
    @Test
    public void testReturnNull() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Object test() {\n" +
                "        return null;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void testPassNull() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Object callee(Object obj) {\n" +
                "        return obj == null;\n" +
                "    }\n" +
                "    public static Object test() {\n" +
                "        return callee(null);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void testAssignNull() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    static String sfield;\n" +
                "    String field;\n" +
                "    public Object test() {\n" +
                "        String local = null;\n" +
                "        this.field = null;\n" +
                "        Test.sfield = null;\n" +
                "        int[][] locArray = null;\n" +
                // local must be null not Interpreternumber(0)
                "        String[][] strings = new String[3][];\n" +
                "        strings[0] = new String[5];\n" +
                "        strings[0][4] = null;\n" +
                "        strings[0][4] = \"test\";\n" +
                "        strings[1] = null;\n" +
                "        return local;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return new Test().test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void testReturnZero() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Object test() {\n" +
                "        return 0;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void testStream() {
        @Language("java") String code = "" +
                "import java.util.Arrays;\n" +
                "public class Test {\n" +
                "    public static Object test() {\n" +
                "        return Arrays.asList(1,2,3).stream().count();\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void interfaceDefault() {
        @Language("java") String code = "" +
                "import java.util.Arrays;\n" +
                "interface iface {\n" +
                "    default String foo() {\n" +
                "        return \"bar\";\n" +
                "    }\n" +
                "}\n" +
                "public class Test implements iface {\n" +
                "    public static Object test() {\n" +
                "        iface obj = new Test();" +
                "        return obj.foo();\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Ljava/lang/Object;").run();
    }

    @Test
    public void testInstanceMethodPatch() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    int a(int val){return val;};\n" +
                "    public static Object test(Object[] args){\n" +
                "        return new Test().a(42);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->a(I)I").run();
    }

    @Test
    public void instanceOfTest1() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        Object obj = 1;\n" +
                "        return obj instanceof Integer;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    @Test
    public void instanceOfTest2() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test() {\n" +
                "        int res = 0;\n" +
                "        Object obj = 1;\n" +
                "        if (obj instanceof Integer) res++;\n" +
                "        if (obj instanceof Long) res++;\n" +
                "        if (obj instanceof Boolean) res++;\n" +
                "        obj = false;\n" +
                "        if (obj instanceof Boolean) res+=10;\n" +
                "        if (obj instanceof Integer) res+=10;\n" +
                "        return res;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void instanceConstructor() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public int a;\n" +
                "    public Test(int a) {\n" +
                "        this.a = a;\n" +
                "    }\n" +
                "    public static Test test(int val) {\n" +
                "        return new Test(val);\n" +
                "    }\n" +
                "    public String toString() {\n" +
                "        return \"\" + a;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test(((Integer) args[0]).intValue()).toString();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)LTest;")
                .addInputSample(5012)
                .run();
    }

    @Test
    public void implicitConstructor() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        return new Test() instanceof Test;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    @Test
    public void objectComparison() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        Test t1 = new Test();\n" +
                "        Test t2 = t1;\n" +
                "        Test t3 = new Test();\n" +
                "        Test t4 = null;\n" +
                "        boolean success = true;\n" +
                "        success &= t1 == t2;\n" +
                "        success &= t1 != t3;\n" +
                "        success &= t4 == null;\n" +
                "        success &= t1 != null;\n" +
                "        success &= t3 != null;\n" +
                "        return success;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    @Test
    public void instanceFieldAccess() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    private int a;\n" +
                "    public Test(int a) {\n" +
                "        this.a = a;\n" +
                "    }\n" +
                "    public static int test(int val) {\n" +
                "        Test t = new Test(val);\n" +
                "        t.a = t.a + t.a;\n" +
                "        return t.a;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I")
                .addInputSample(5)
                .run();
    }

    @Test
    public void inheritedFieldAccess() {
        @Language("java") String code = "" +
                "class A {\n" +
                "    int a;\n" +
                "}\n" +
                "public class Test extends A {\n" +
                "    public static int test(int val){\n" +
                "        Test b = new Test();\n" +
                "        b.a = val;\n" +
                "        return b.a;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I")
                .addInputSample(5)
                .run();
    }

    @Test
    public void staticInnerClass() {
        @Language("java") String code = "" +
                "class A {\n" +
                "    public static class B {\n" +
                "        int b1;\n" +
                "        B (int val) {\n" +
                "            b1 = val;\n" +
                "        }\n" +
                "    }\n" +
                "}\n" +
                "public class Test {\n" +
                "    public static int test(int val) {\n" +
                "        A.B b = new A.B(val);\n" +
                "        return b.b1;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test(42);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I").run();
    }

//    the super instruction is currently not supported
//    @Test
//    public void callSuper() {
//        @Language("java") String code = "" +
//                "class A {\n" +
//                "    int val = 3;\n" +
//                "    protected int method(int a, int b) {\n" +
//                "        return val + a + b + 2;\n" +
//                "    }\n" +
//                "    protected double methodArgs(int a, double b, int c, long d, float e, byte f, long g) {\n" +
//                "        return val + a + b + c + d + e + f + g;\n" +
//                "    }\n" +
//                "}\n" +
//                "\n" +
//                "public class Test extends A {\n" +
//                "    public int method(int a, int b){\n" +
//                "        return 1;\n" +
//                "    }\n" +
//                "    protected double methodArgs(int a, double b, int c, long d, float e, byte f, long g) {\n" +
//                "        return 2.;\n" +
//                "    }\n" +
//                "\n" +
//                "    public int call(int a, int b){return method(a, b);}\n" +
//                "    public int callSuper(int a, int b){return super.method(a, b);}\n" +
//                "    public double callArgs(int a, double b, int c, long d, float e, byte f, long g){return methodArgs(a, b, c, d, e, f, g);}\n" +
//                "    public double callArgsSuper(int a, double b, int c, long d, float e, byte f, long g){return super.methodArgs(a, b, c, d, e, f, g);}\n" +
//                "    public static void test(){\n" +
//                "        Test b = new Test();\n" +
//                "        if (b.call(1,2) != 1) throw new RuntimeException(\"something strange happened\");\n" +
//                "        if (b.callSuper(1,2) != 8) throw new RuntimeException(\"something strange happened2\");\n" +
//                "        if (b.callArgs(1,2,3,4,5,(byte) 6,7) != 2.) throw new RuntimeException(\"something strange happened3\");\n" +
//                "        if (b.callArgsSuper(1,2,3,4,5,(byte) 6,7) != 31.) throw new RuntimeException(\"something strange happened4\");\n" +
//                "    }\n" +
//                "    public static Object test(Object[] args){\n" +
//                "        test();\n" +
//                "        return nu" +
//                "ll;\n" +
//                "    }\n" +
//                "}";
//        new InterpreterTestCase(code, "LTest;->test()V", "LTest;->callSuper(II)I", "LTest;->call(II)I", "LTest;->method(II)I", "LA;->method(II)I",
//                "LTest;->callArgsSuper(IDIJFBJ)D", "LTest;->callArgs(IDIJFBJ)D", "LTest;->methodArgs(IDIJFBJ)D", "LA;->methodArgs(IDIJFBJ)D").run();
//    }

    @Test
    public void fieldAccessTypes() {
        @Language("java") String code = "" +
                "import java.util.Random;\n" +
                "public class Test {\n" +
                "    Object objMember;\n" +
                "    boolean booleanMember;\n" +
                "    byte byteMember;\n" +
                "    char charMember;\n" +
                "    short shortMember;\n" +
                "    int intMember;\n" +
                "    long longMember;\n" +
                "    float floatMember;\n" +
                "    double doubleMember;\n" +
                "\n" +
                "    public Test(Object objMember, boolean booleanMember, byte byteMember, char charMember, short shortMember, int intMember, long longMember, float floatMember, double doubleMember) {\n" +
                "        this.objMember = objMember;\n" +
                "        this.booleanMember = booleanMember;\n" +
                "        this.byteMember = byteMember;\n" +
                "        this.charMember = charMember;\n" +
                "        this.shortMember = shortMember;\n" +
                "        this.intMember = intMember;\n" +
                "        this.longMember = longMember;\n" +
                "        this.floatMember = floatMember;\n" +
                "        this.doubleMember = doubleMember;\n" +
                "    }\n" +
                "\n" +
                "    public static boolean test() {\n" +
                "        Random rand = new Random();\n" +
                "        Object objVal = new String(\"MyString\");\n" +
                "        boolean booleanVal = rand.nextBoolean();\n" +
                "        byte byteVal = (byte) rand.nextInt();\n" +
                "        char charVal = (char) rand.nextInt();\n" +
                "        short shortVal = (short) rand.nextInt();\n" +
                "        int intVal = rand.nextInt();\n" +
                "        long longVal = rand.nextLong();\n" +
                "        float floatVal = rand.nextFloat();\n" +
                "        double doubleVal = rand.nextDouble();\n" +
                "        Test obj = new Test(objVal, booleanVal, byteVal, charVal, shortVal, intVal, longVal, floatVal, doubleVal);\n" +
                "        boolean success = obj == obj.objMember;\n" +
                "        success &= booleanVal == obj.booleanMember;\n" +
                "        success &= byteVal == obj.byteMember;\n" +
                "        success &= charVal == obj.charMember;\n" +
                "        success &= shortVal == obj.shortMember;\n" +
                "        success &= intVal == obj.intMember;\n" +
                "        success &= longVal == obj.longMember;\n" +
                "        success &= floatVal == obj.floatMember;\n" +
                "        success &= doubleVal == obj.doubleMember;\n" +
                "        return success;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        // TODO also patch constructor when this is supported
//        new InterpreterTestCase(code, "LTest;->test()Z", "LTest;-><init>(Ljava/lang/Object;ZBCSIJFD)V").run();
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    // TODO support constructor patching
//    @Test
//    public void constructorPatching() {
//        @Language("java") String code = "" +
//                "public class Test {\n" +
//                "    int val;\n" +
//                "    public Test(int val){\n" +
//                "        this.val = val;\n" +
//                "    }\n" +
//                "    public static int test(int i){\n" +
//                "        return new Test(i).val;\n" +
//                "    }\n" +
//                "    public static Object test(Object[] args){\n" +
//                "        return test(42);\n" +
//                "    }\n" +
//                "}";
//        new InterpreterTestCase(code, "LTest;->test(I)I", "LTest;-><init>(I)V").run();
//    }
//
//    @Test
//    public void constructorPatchingComplex() {
//        @Language("java") String code = "" +
//                "class Base {\n" +
//                "    String bval;\n" +
//                "    public Base(String bval) {\n" +
//                "        this.bval = bval;\n" +
//                "    }\n" +
//                "}\n" +
//                "class Sub extends Base{\n" +
//                "    int sval;\n" +
//                "    public Sub(int sval) {\n" +
//                "        super(new StringBuilder(\"Complex \").append(sval).toString());\n" +
//                "        this.sval = sval;\n" +
//                "    }\n" +
//                "    public Sub() {\n" +
//                "        this(3);\n" +
//                "    }\n" +
//                "    public static String test(int i) {\n" +
//                "        return new Sub().bval;\n" +
//                "    }\n" +
//                "    public static Object test(Object[] args){\n" +
//                "        return test(42);\n" +
//                "    }\n" +
//                "}";
//        new InterpreterTestCase(code, "LSub;->test(I)Ljava/lang/String;", "LSub;-><init>()V", "LSub;-><init>(I)V", "LBase;-><init>(Ljava/lang/String;)V").run();
//    }


    @Test
    public void inheritedMethodAccess() {
        @Language("java") String code = "" +
                "class A {\n" +
                "    int a(int val){return val;};\n" +
                "}\n" +
                "public class Test extends A {\n" +
                "    public static int test(int val){\n" +
                "        Test b = new Test();\n" +
                "        return b.a(val);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test(5);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I", "LA;->a(I)I").run();
    }

    @Test
    public void inheritedFields(){
        @Language("java") String code = "" +
                "class A {\n" +
                "    public static final String STRING_S_CONST = \"MY_CONST\";\n" +
                "    public static int INT_S_FIELD = 3;\n" +
                "    public int INT_I_FIELD = 4;\n" +
                "}\n" +
                "public class Test extends A {\n" +
                "    public static int test(){\n" +
                "        Test obj = new Test();\n" +
                "        return STRING_S_CONST.length() + INT_S_FIELD + obj.INT_I_FIELD;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void objectCast() {
        @Language("java") String code = "" +
                "class A {\n" +
                "    int a(int val){return val;};\n" +
                "}\n" +
                "public class Test extends A {\n" +
                "    public static int test(int val){\n" +
                "        Object obj = new Test();\n" +
                "        A a = (A) obj;\n" +
                "        Test b = (Test) a;\n" +
                "        a = (A) obj;\n" +
                "        return a.a(val);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test(5);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I", "LA;->a(I)I").run();
    }

    @Test
    public void objectInvalidCast() {
        @Language("java") String code = "" +
                "class A {\n" +
                "    int a(int val){return val;};\n" +
                "}\n" +
                "public class Test extends A {\n" +
                "    public static boolean test(){\n" +
                "        Object obj = new A();\n" +
                "        try{\n" +
                "            Test b = (Test) obj;\n" +
                "            return true;\n" +
                "        } catch (java.lang.ClassCastException e) {\n" +
                "            return false;\n" +
                "        }\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z", "LA;->a(I)I").run();
    }
}
