package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

import java.util.Random;

public class PerformanceTest {
    @Test
    public void bubbleSort() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static Comparable[] bubbleSort(Comparable... arr) {\n" +
                "        int n = arr.length;\n" +
                "        Comparable temp = null;\n" +
                "        for(int i=0; i < n; i++){\n" +
                "            for(int j=1; j < (n-i); j++){\n" +
                "                if(arr[j-1].compareTo(arr[j]) > 0){\n" +
                "                    temp = arr[j-1];\n" +
                "                    arr[j-1] = arr[j];\n" +
                "                    arr[j] = temp;\n" +
                "                }\n" +
                "            }\n" +
                "        }\n" +
                "        return arr;\n" +
                "    }" +
                "    public static Object test(Object[] args){\n" +
                "        return bubbleSort((Comparable[]) args);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->bubbleSort([Ljava/lang/Comparable;)[Ljava/lang/Comparable;")
                .addInputSample(new Random().ints(500, Integer.MIN_VALUE, Integer.MAX_VALUE).boxed().toArray(Integer[]::new))
                .addInputSample(new Random().ints(1000, Integer.MIN_VALUE, Integer.MAX_VALUE).boxed().toArray(Integer[]::new))
                .run();
    }

    @Test
    public void testIterateLoop() {
        @Language("java" +
                "") String code = "" +
                "public class Test {\n" +
                "    public static long loop(int num) {\n" +
                "        long res = 0;" +
                "        for (int i = 0; i < num; i++) {\n" +
                "            res += i;\n" +
                "        }\n" +
                "        return res;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return loop((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->loop(I)J")
                .addInputSample(1000)
                .addInputSample(10000)
                .addInputSample(100000)
                .addInputSample(1000000)
                .addInputSample(10000000)
                .run();
    }

    @Test
    public void testRecFib() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int fib(int num) {\n" +
                "       if(num < 1) return 0;\n" +
                "       if(num == 1) return 1;\n" +
                "       return fib(num - 1) + fib(num - 2);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return fib((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->fib(I)I")
                .addInputSample(20)
                .addInputSample(30)
                .run();
    }

    @Test
    public void quickSortJ8() {
        @Language("java") String code = "" +
                "import java.util.*;\n" +
                "import java.util.function.BiPredicate;\n" +
                "import java.util.stream.Collectors;\n" +
                "import java.util.stream.Stream;\n" +
                "import static java.util.stream.Collectors.partitioningBy;" +
                "\n" +
                "public class Test {\n" +
                "    public static <T> List<T> qs(List<T> l, BiPredicate<T, T> isLower) {\n" +
                "        if (l.size() < 2) {\n" +
                "            return l;\n" +
                "        } else {\n" +
                "            T x = l.get(0);\n" +
                "            Stream<T> xs = l.stream().skip(1);\n" +
                "            Map<Boolean, List<T>> part = xs.collect(partitioningBy(item -> isLower.test(item, x)));\n" +
                "            List<T> lowers = part.get(true);\n" +
                "            List<T> highers = part.get(false);\n" +
                "            return Stream.concat(Stream.concat(qs(lowers, isLower).stream(), Stream.of(x)), qs(highers, isLower).stream()).collect(Collectors.toList());\n" +
                "        }\n" +
                "    }\n" +
                "    public static Object test(Object[] args) {\n" +
                "        return qs(Arrays.asList((Integer[]) args), (a, b) -> a < b);\n\n" +
                "    }\n" +
                "}";
        // TODO also patch lambdas to use interpreter
        new InterpreterTestCase(code, InterpreterTestCase.DVM_EXEC, "LTest;->qs(Ljava/util/List;Ljava/util/function/BiPredicate;)Ljava/util/List;")
                .addInputSample(new Random().ints(500, Integer.MIN_VALUE, Integer.MAX_VALUE).boxed().toArray(Integer[]::new))
                .addInputSample(new Random().ints(1000, Integer.MIN_VALUE, Integer.MAX_VALUE).boxed().toArray(Integer[]::new))
                .run();
    }
}
