package de.marstra.dvo.obfuscator.obfuscator;

import org.intellij.lang.annotations.Language;
import org.junit.Test;

import java.util.Random;

public class PrimitiveTypesTest {
    @Test
    public void testReturnFalse() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        return false;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }

    @Test
    public void testReturnTrue() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean test() {\n" +
                "        return true;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()Z").run();
    }


    @Test
    public void testCallBoolArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static boolean callee(boolean arg) {\n" +
                "       return arg;\n" +
                "    }\n" +
                "    public static int test(int val){\n" +
                "        return callee(val > 3) ? 5 : 3;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";

        new InterpreterTestCase(code, "LTest;->test(I)I", "LTest;->callee(Z)Z")
                .addInputSample(4)
                .addInputSample(3)
                .addInputSample(2)
                .run();
    }

    @Test
    public void testReturnChar() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static char test() {\n" +
                "        return 'A';\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()C").run();
    }

    @Test
    public void testReturnArgChar() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static char test(char c) {\n" +
                "        return (char) (c + 3);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Character) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(C)C")
                .addInputSample('A')
                .addInputSample((char) new Random().nextInt())
                .run();
    }

    @Test
    public void testCallCharArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static char callee(char c) {\n" +
                "       return (char) (c + 3);\n" +
                "    }\n" +
                "    public static char test(char c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Character) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(C)C", "LTest;->callee(C)C")
                .addInputSample('A')
                .addInputSample((char) new Random().nextInt())
                .run();
    }

    @Test
    public void testReturnByte() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static byte test() {\n" +
                "        return 3;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()B").run();
    }

    @Test
    public void testReturnArgByte() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static byte test(byte b) {\n" +
                "        return (byte) (b + 250);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Byte) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(B)B")
                .addInputSample((byte) 10)
                .addInputSample((byte) new Random().nextInt())
                .run();
    }

    @Test
    public void testCallByteArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static byte callee(byte b) {\n" +
                "       return (byte) (b + 250);\n" +
                "    }\n" +
                "    public static byte test(byte c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Byte) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(B)B", "LTest;->callee(B)B")
                .addInputSample((byte) 10)
                .addInputSample((byte) new Random().nextInt())
                .run();
    }

    @Test
    public void testReturnShort() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static short test() {\n" +
                "        return (short) 0xFFFF;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()S").run();
    }

    @Test
    public void testReturnArgShort() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static short test(short b) {\n" +
                "        return (short) (b + 0x7FFE);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Short) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(S)S")
                .addInputSample((short) 10)
                .addInputSample((short) new Random().nextInt())
                .run();
    }

    @Test
    public void testCallShortArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static short callee(short b) {\n" +
                "       return (short) (b - 0xFFFE);\n" +
                "    }\n" +
                "    public static short test(short c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Short) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(S)S", "LTest;->callee(S)S")
                .addInputSample((short) 10)
                .addInputSample((short) new Random().nextInt())
                .run();
    }

    @Test
    public void testReturnInt() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test() {\n" +
                "        return 0x7FFFFFFF;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()I").run();
    }

    @Test
    public void testReturnArgInt() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int test(int b) {\n" +
                "        return b * 17;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I")
                .addInputSample(42)
                .addInputSample(new Random().nextInt())
                .run();
    }

    @Test
    public void testCallIntArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static int callee(int b) {\n" +
                "       return b - 38;\n" +
                "    }\n" +
                "    public static int test(int c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Integer) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(I)I", "LTest;->callee(I)I")
                .addInputSample(42)
                .addInputSample(new Random().nextInt())
                .run();
    }

    @Test
    public void testReturnFloat() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static float test() {\n" +
                "        return 3.75f;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()F").run();
    }

    @Test
    public void testReturnArgFloat() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static float test(float b) {\n" +
                "        return b * 3.75f;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Float) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(F)F")
                .addInputSample(1.4221f)
                .addInputSample(new Random().nextFloat())
                .run();
    }

    @Test
    public void testCallFloatArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static float callee(float b) {\n" +
                "       return b - 1924.8213f;\n" +
                "    }\n" +
                "    public static float test(float c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Float) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(F)F", "LTest;->callee(F)F")
                .addInputSample(42.424242f)
                .addInputSample(new Random().nextFloat())
                .run();
    }

    @Test
    public void testReturnDouble() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double test() {\n" +
                "        return 1.3e7;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()D").run();
    }

    @Test
    public void testReturnArgDouble() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double test(double b) {\n" +
                "        return b * 3.75;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Double) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(D)D")
                .addInputSample(1.4221)
                .addInputSample(new Random().nextDouble())
                .run();
    }

    @Test
    public void testCallDoubleArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static double callee(double b) {\n" +
                "       return b - 1924.8213;\n" +
                "    }\n" +
                "    public static double test(double c) {\n" +
                "        return callee(c);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Double) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(D)D", "LTest;->callee(D)D")
                .addInputSample(42.424242)
                .addInputSample(new Random().nextDouble())
                .run();
    }

    @Test
    public void testReturnLong() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long test() {\n" +
                "        return 0x4242424242424242L;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test();\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test()J").run();
    }

    @Test
    public void testReturnArgLong() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long test(long b) {\n" +
                "        return b * 42L;\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Long) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(J)J")
                .addInputSample(42L)
                .addInputSample(new Random().nextLong())
                .run();
    }

    @Test
    public void testCallLongArg() {
        @Language("java") String code = "" +
                "public class Test {\n" +
                "    public static long callee(long b, long c) {\n" +
                "       return b - 0xFFFFFFFFFEL + c;\n" +
                "    }\n" +
                "    public static long test(long c) {\n" +
                "        return callee(c, 3);\n" +
                "    }\n" +
                "    public static Object test(Object[] args){\n" +
                "        return test((Long) args[0]);\n" +
                "    }\n" +
                "}";
        new InterpreterTestCase(code, "LTest;->test(J)J", "LTest;->callee(JJ)J")
                .addInputSample(424242L)
                .addInputSample(new Random().nextLong())
                .run();
    }

    // TODO test invoke-range(long,long,...)

    // TODO test primitve arrays
}
