#!/usr/bin/env python
import argparse
import subprocess
from os.path import exists, isfile
from os import mkdir, remove
import shutil

JAVAC = "javac"
DX = "~/Android/Sdk/build-tools/28.0.2/dx"
BAKSAMLI = "~/repos/smali/scripts/baksmali"
DEXDUMP = "~/Android/Sdk/build-tools/28.0.2/dexdump"

CLASSES_DIR = "classes"
SMALI_DIR = "smali"
DEXDUMP_FILE = "classes.dexdump"
DEX_FILE ="classes.dex"

CLI_ARGS = None

def call(args):
    return subprocess.check_output(args, shell=True)

def main():
    if CLI_ARGS.clean:
        for output in [CLASSES_DIR, SMALI_DIR, DEXDUMP_FILE, DEX_FILE]:
            if isfile(output):
                remove(output)
            else:
                shutil.rmtree(output, ignore_errors=True)

    if not CLI_ARGS.sources:
        return

    for output in [CLASSES_DIR, SMALI_DIR, DEXDUMP_FILE, DEX_FILE]:
        if exists(output):
            raise Exception(output + " already exists")

    mkdir(CLASSES_DIR)
    call(JAVAC + " -d " + CLASSES_DIR + " " + " ".join(CLI_ARGS.sources))

    call(DX + " --dex --output=" + DEX_FILE + " " + CLASSES_DIR)

    dump = call(DEXDUMP + " -d " + DEX_FILE)
    with open(DEXDUMP_FILE, "w") as f:
        f.write(dump)

    call(BAKSAMLI + " d " + DEX_FILE + " -o " + SMALI_DIR)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Compile Java to Dalvik Bytecode")
    parser.add_argument("sources", nargs="*", help="Java source files")
    parser.add_argument("-c", "--clean", action="store_true", help="clean output")
    CLI_ARGS = parser.parse_args()
    main()
