#!/usr/bin/env python
import argparse
import os
import os.path as path
import shutil
import subprocess
import tempfile
from xml.dom import minidom

def call(args):
    return subprocess.check_call(args, shell=True)


def writeFile(path, content):
    with open(path, "w") as f:
        return f.write(content)


def adjustManifest(manifestPath, suffix):
    manifest = minidom.parse(manifestPath)

    manifestEles = manifest.getElementsByTagName("manifest")

    if len(manifestEles) != 1:
        raise RuntimeError("invalid number of manifest tags in AndroidManifest.xml")

    manifestEle = manifestEles[0]

    package = manifestEle.getAttribute("package")

    if package is None:
        raise RuntimeError("package not found in AndroidManifest.xml")

    newPackage = package + "_" + suffix.replace(" ", "").lower()
    manifestEle.setAttribute("package", newPackage)

    def traverseNode(node):
        if node.attributes:
            for attrName in node.attributes.keys():
                attrValue = node.attributes[attrName].nodeValue
                if attrValue.startswith("."):
                    newVal = package + attrValue
                    print "patch attribute " + attrName + " of value " + attrValue + " to value " + newVal
                    node.setAttribute(attrName, newVal)

        if node.childNodes:
            for child in node.childNodes:
                traverseNode(child)

    traverseNode(manifestEle)

    writeFile(manifestPath, manifest.toxml().encode('utf-8'))


def adjustStrings(resPath, suffix):
    for root, directories, filenames in os.walk(resPath):
        for filename in filenames:
            if filename == "strings.xml":
                filePath = os.path.join(root, filename)
                stringDom = minidom.parse(filePath)

                stringEles = stringDom.getElementsByTagName("string")
                for stringEle in stringEles:
                    name = stringEle.getAttribute("name")

                    if name == "app_name":
                        appName = stringEle.firstChild.nodeValue
                        newAppName = appName + " " + suffix
                        print "patch appname from " + appName + " to " + newAppName
                        stringEle.firstChild.nodeValue = newAppName

                writeFile(filePath, stringDom.toxml().encode('utf-8'))


def renameApp(inputApk, outputApk, suffix):
    if not path.exists(inputApk):
        raise RuntimeError("inputApk does not exist: " + inputApk)

    workDir = path.join(tempfile.gettempdir(), "rename_app_work")
    if path.exists(workDir):
        shutil.rmtree(workDir)

    os.mkdir(workDir)
    shutil.copy(inputApk, workDir)

    inputApk = path.join(workDir, path.basename(inputApk))
    unpackedApk = path.join(workDir, "unpackedApk")

    call("apktool d -o " + unpackedApk + " " + inputApk)

    adjustManifest(path.join(unpackedApk, 'AndroidManifest.xml'), suffix)
    adjustStrings(path.join(unpackedApk, "res"), suffix)

    call("apktool b " + unpackedApk)

    shutil.copy(path.join(unpackedApk, "dist", path.basename(inputApk)), outputApk)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Appends suffix to an android apps name")
    parser.add_argument("inputApk", help="apk to patch")
    parser.add_argument("outputApk", help="resulting apk")
    parser.add_argument("suffix", help="suffix to add to app name")
    args = parser.parse_args()
    renameApp(args.inputApk, args.outputApk, args.suffix)
