package de.marstra.dvo.webservice.resources;

import com.fasterxml.jackson.annotation.JsonProperty;
import de.marstra.dvo.obfuscator.Config;
import de.marstra.dvo.obfuscator.ObfuscatorService;
import de.marstra.dvo.obfuscator.analysis.*;
import de.marstra.dvo.obfuscator.util.GenericMethodReference;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import java.util.Base64;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

@Data
class ObfuscationRequest {
    @JsonProperty
    private Config config;
    @JsonProperty
    private String apkFile;
}

@Slf4j
@Path("/")
public class ObfuscateResource {
    @Context
    private HttpServletRequest request;

    @POST
    @Path("obfuscate")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces("application/vnd.android.package-archive")
    public byte[] obfuscate(ObfuscationRequest obfuscationRequest) {
        log.info("obfuscate: " + obfuscationRequest.getApkFile().substring(0, 100));
        final Config config = obfuscationRequest.getConfig();
        ApkFile apkFile = new ApkFile(Base64.getDecoder().decode(obfuscationRequest.getApkFile()));
        return ObfuscatorService.obfuscate(apkFile, config);
    }

    @POST
    @Path("analyseapk")
    @Produces(MediaType.APPLICATION_JSON)
    public Map analyseapk(String b64apk) {
        log.info("analyseapk: " + b64apk.substring(0, 100));
        ApkFile apkFile = new ApkFile(Base64.getDecoder().decode(b64apk));

        Map<String, Integer> errorDistribution = new HashMap<>();
        ParsedDex parsedDex = apkFile.getParsedDex();
        Map apkTree = parsedDex.accept(new AnalysisVisitorAdapter<Map, Void>() {
            @Override
            public Map visit(DexMethod dexMethod, Void p) {
                HashMap<String, Object> jsonObj = new HashMap<>();
                jsonObj.put("type", "method");
                jsonObj.put("gmr", new GenericMethodReference(dexMethod.getMethod()));
                MethodAnalysis.MethodAnalysisResult analysisResult = MethodAnalysis.analyseMethod(dexMethod.getMethod(), parsedDex);
                jsonObj.put("analysisResult", analysisResult);
                for (String reason : analysisResult.getUnsupportedReasons()) {
                    errorDistribution.putIfAbsent(reason, 0);
                    errorDistribution.put(reason, errorDistribution.get(reason) + 1);
                }
                return jsonObj;
            }

            @Override
            protected Map aggregateClass(DexClass dexClass, Map classJson, Map method) {
                if (classJson == null) {
                    classJson = new HashMap();
                    classJson.put("type", "class");
                    classJson.put("name", dexClass.getName());
                    classJson.put("numMethods", 0);
                    classJson.put("numUnsupportedMethods", 0);
                    classJson.put("children", new LinkedList<>());
                }
                ((LinkedList) classJson.get("children")).add(method);
                classJson.put("numMethods", (Integer) classJson.get("numMethods") + 1);
                if (!((MethodAnalysis.MethodAnalysisResult) method.get("analysisResult")).isSupported())
                    classJson.put("numUnsupportedMethods", (Integer) classJson.get("numUnsupportedMethods") + 1);
                return classJson;
            }

            @Override
            protected Map aggregatePackage(DexPackage dexPackage, Map packageJson, Map child) {
                if (packageJson == null) {
                    packageJson = new HashMap();
                    String name = dexPackage.getName();
                    if (name == null)
                        name = "root";
                    packageJson.put("type", "package");
                    packageJson.put("name", name);
                    packageJson.put("numMethods", 0);
                    packageJson.put("numUnsupportedMethods", 0);
                    packageJson.put("children", new LinkedList<>());
                }
                if(child != null) {
                    // the case when classes have no methods
                    ((LinkedList) packageJson.get("children")).add(child);
                    for (String key : new String[]{"numMethods", "numUnsupportedMethods"})
                        packageJson.put(key, (Integer) packageJson.get(key) + (Integer) child.get(key));
                }
                return packageJson;
            }
        });

        Map<String, Object> resultJson = new HashMap<>();
        resultJson.put("parsedApk", apkTree);
        resultJson.put("errorDistribution", errorDistribution);
        log.info("analysedapk: " + b64apk.substring(0, 100));
        return resultJson;
    }
}
